package edu.uoc.framework.action;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;

/**
 * <p>Interfaz que define el contrato que debe implementar una Acción.</p>
 * 
 * La implementación de un Action en JavaMVC representa una clase de soporte del controlador encargada de:
 * <ul><li>Recibir las peticiones de los usuarios e invocar a lógica de negocio.</li>
 * <li>Determinar a qué recurso (Vista u otro Action) enviar los resultados de la petición.</li>
 * </ul>
 * 
 * <p>Para poder utilizar un Action es necesario haber declarado éste en el XML de configuración
 * del framework.</p>
 *
 *<p>Ejemplo de declaración de un Action</p>
 * <pre>
 *{@code 
 *<jmvc:action path="Login" class="edu.uoc.presentation.actions.Login" actionForm="LoginForm" input="/jsp/login.jsp">
 *	<jmvc:interceptor-ref name="interceptor1"/>
 *	<jmvc:forward name="ok" path="/jsp/..."/>
 *	<jmvc:forward name="error" path="/jsp/..." redirect="true"/>
 *</jmvc:action>}
 * </pre>
 * 
 * <p>Elementos de un Action</p>
 * <ul><li>name - Atributo obligatorio que indica el nombre del Action. Éste debe coincidir con el nombre del recurso solicitado en la petición HTTP.
 * <br>Ejemplo: http://localhost:8080/test/Login.do</li>
 * <li>class - Atributo obligatorio que indica la clase que implementará Action.</li>
 * <li>actionForm - Atributo opcional utilizado cuando, vinculado a un Action, existe un formulario de entrada de datos.<br>
 * Este campo tendrá el nombre del {@link Form} que permitirá realizar validaciones sobre los datos recibidos.</li>
 * <li>input - Atributo opcional que indica la página que envía el formulario de datos al Action y que además será la que mostrará los errores de validación si los datos recibidos no son correctos.</li>
 * </ul>
 * 
 * <p>Todo Action tendrá asociados uno o más recursos de destino 'forward' y podrá tener vinculados varios interceptores.<p>
 * <p>Si un Action utiliza interceptores, éstos se irán ejecutando según el órden de aparición en la declaráción.</p>
 * 
 * <p>Dado que los interceptores de un Action son realmente referencias, la definición completa de cada Interceptor deberá haberse declarado previamente tal y como se indica
 * en {@link edu.uoc.framework.interceptor.Interceptor}.
 * 
 * <p>Un forward tendrá los siguientes atributos:</p>
 * 		<ul><li>name - Atributo obligatorio que identificará al forward y que el Action utilizará para indicar a quíen enviar el resultado de su ejecución.</li> 
 * 		<li>path - Atributo obligatorio que indica el path del recurso al que se desea enviar el resultado de la ejecución del Action.</li>
 * 		<li>redirect - Atributo opcional que indicará al framework que cuando tenga que despacharse el recurso con nombre 'name' deberá
 * 		hacerse mediante una redirección en lugar de mediante un forward.</li></ul>
 * 
 * @author Alejandro Marmelo Insua
 * 
 */
public interface Action {

	/**
	 * <p>Método al que despachará el controlador con el objetivo de llevar a cabo la lógica vinculada con la petición.</p>
	 * <p>La lógica asociada a este método debe centrarse en realizar las
	 * llamadas a los métodos de dominio necesarios para llevar a cabo la lógica de
	 * negocio y determinar a qué recurso (Vista u otro Action) despachar el resultado.</p>
	 * 
	 * @param contextObject objeto que encapsula los datos de contexto de la petición recibida.
	 * @param form formulario vinculado al Action que contendrá los datos recuperados
	 * 				 del formulario de entrada. Este parámetro será null si el 
	 * 				 Action no tiene {@link edu.uoc.framework.form.Form} asociado.
	 * @return Instancia de {@link ActionResponse} en la que se incluirán tanto el recurso
	 * 			destino de la petición como los datos que éste necesite.
	 * @throws Exception Excepción que representa una situación de error sucedida durante la ejecución del método.
	 */
	public ActionResponse execute(ContextObject contextObject, Form form) throws Exception;
}
