package edu.uoc.framework.action;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpSession;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.I18N.Message;

/**
 * <p>Clase abstracta que implementa {@link Action} 
 * y proporciona un conjunto de métodos de soporte
 * que facilitan la implementación de una Acción.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public abstract class ActionSupport implements Action 
{
	/**
	 * Propiedad que almacena los mensajes de error.
	 */
	private FormErrors messages = null;
	
	/**
	 * Propiedad que recoge el objeto contexto.
	 */
	private ContextObject contextObject = null;
	
	/**
	 * Propiedad que recoge el formulario vinculado con el Action
	 */
	private Form form = null;
	
	/**
	 * <p>Método que recupera el objeto Form asociado al Action.</p>
	 * 
	 * @return {@link edu.uoc.framework.form.Form}
	 */
	public final Form getForm() {
		return form;
	}
	
	/**
	 * <p>Método que recupera el ContextObject asociado al Action.</p>
	 * @return {@link edu.uoc.framework.core.ContextObject}
	 */
	public final ContextObject getContextObject() {
		return contextObject;
	}
	
	/**
	 * <p>Método que permite añade mensajes de error internacionalizados a la Acción</p>
	 * 
	 * <p>Estos mensajes de error llegarán a la Vista y podrán recuperarse a partir de su clave.</p>
	 * 
	 * @param key Identificador que se asocia el mensaje internacionalizado.
	 * @param message Instancia de {@link edu.uoc.framework.utilities.I18N.Message}
	 */
	public final void addErrorMessage (String key, Message message) {
		messages.addError(key, message);
	}
	
	/**
	 * <p>Método que permite recuperar un parámetro de la request a partir de su nombre.</p>
	 * 
	 * @param name Nombre del parámetro que quiere recuperarse de la request.
	 * @return valor {@link java.lang.String} asociado al parámetro 'name' de la request.
	 */
	public final String getRequestParameter (String name) {
		return contextObject.getRequest().getParameter(name);
	}
	
	/**
	 * <p>Método que permite establecer un parámetro en la request</p>
	 * 
	 * @param name Nombre del parámetro al que se quiere asignar el contenido 'value'.
	 * @param value Contenido que quiere asociarse con el parámetro 'name' en la request. 
	 */
	public final void setRequestAttribute (String name, Object value) {
		contextObject.getRequest().setAttribute(name, value);
	}
	
	/**
	 * Método que permite recuperar el objeto sesión de la petición
	 * @return {@link HttpSession}
	 */
	public final HttpSession getSession() {
		return contextObject.getRequest().getSession();
	}
	
	/**
	 * Método que permite recuperar el objeto sesión de la petición y si 
	 * createSession vale true, en caso de no existir sesión crearla.
	 * 
	 * @param createSession booleano que indica si la sesión debe crearse o no en caso de no existir.
	 * @return {@link HttpSession}
	 */
	public final HttpSession getSession(boolean createSession) {
		return contextObject.getRequest().getSession(createSession);
	}
	
	/**
	 * Método que permite recuperar el valor de un atributo de la sesión.
	 * 
	 * @param attribute Nombre del atributo de sesión.
	 * @return Valor asociado al atributo de sesión indicado por 'attribute'.
	 */
	public final Object getSessionAttribute(String attribute) {
		Object value = null;
		if(getSession()!=null)
			value = getSession().getAttribute(attribute);
		return value;
	}
	
	/**
	 * Método que permite establecer en sesión un atributo de nombre
	 * 'attribute' y con valor 'value'.
	 * 
	 * @param attribute Nombre del atributo que quiere definirse en sesión.
	 * @param value Valor a asignar al atributo de sesión 'attribute'.
	 */
	public final void setSessionAttribute(String attribute, Object value)
	{
		HttpSession session = null;
		if ((session = getSession()) != null)
			session.setAttribute(attribute, value);
	}
	
	public final ActionResponse execute(ContextObject contextObject, Form form)
	throws Exception
	{
		ActionResponse actionResponse = null;
		Map<String,Object> responseData = null;
		
		this.contextObject = contextObject;
		this.form = form;
		this.messages = new FormErrors();
		
		actionResponse = actionExecute();
		
		if (!messages.isEmpty())
		{
			if ((responseData = actionResponse.getData())!=null)
				responseData = actionResponse.getData();
			else
				responseData = new HashMap<String, Object>();
			
			responseData.put(Constants.JavaMVC_ErrorMessages, messages);
			actionResponse = new ActionResponse(responseData, actionResponse.getResource());
		}
		return actionResponse;
	}

	/**
	 * Método a implementar por las clases que utilicen ActionSupport.
	 * 
	 * @return {@link ActionResponse}
	 * @throws Exception Excepción sucedida durante la ejecución de la Acción.
	 */
	public abstract ActionResponse actionExecute() throws Exception;
}
