package edu.uoc.framework.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.Action;
import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.core.configuration.Forward;
import edu.uoc.framework.core.configuration.InterceptorRef;
import edu.uoc.framework.exception.ActionException;
import edu.uoc.framework.exception.FormException;
import edu.uoc.framework.exception.InterceptorException;
import edu.uoc.framework.exception.ViewException;
import edu.uoc.framework.exceptionHandler.ExceptionHandler;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.form.FormErrors;
import edu.uoc.framework.helpers.Data;
import edu.uoc.framework.helpers.ResourceAndData;
import edu.uoc.framework.interceptor.Interceptor;
import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.ReflectionUtilities;
import edu.uoc.framework.view.ViewUtils;

/**
 * <p>Clase que implementa el patrón ApplicationController y Singleton consiguiendo; 
 * por una parte separar del FrontController la gestión de invocación a las acciones
 * y la gestión de envío a las vistas y por otra, proporcionando una única instancia
 * encargada de procesar todas las peticiones.</p>
 *
 * <p>ApplicationController dispone únicamente de un método público que recibe los datos del contexto
 * y el mapeo vinculado con la petición en curso. Con ellos éste método seguirá el flujo declarado en
 * el fichero de configuración del framework.</p>
 * 
 * @author Alejandro Marmelo Insua
 */
public class ApplicationController 
{
	private static Logger log = Logger.getLogger(ApplicationController.class);
	
	private static ApplicationController INSTANCE = null;
	
	private static void createInstance() 
	{
		if (INSTANCE == null) 
		{
			synchronized(ApplicationController.class) 
	        {
				if (INSTANCE == null) 
					INSTANCE = new ApplicationController();
	        }
		}
	}
	
	/**
	 * Método que recupera la única instancia disponible de ApplicacionController.
	 * @return {@link ApplicationController}
	 */
	public static ApplicationController getInstance() 
	{
		if (INSTANCE == null)
			createInstance();
		return INSTANCE;
	}
	
	/**
	 * <p>Método encargado de procesar una petición siguiendo la configuración
	 * declarada para ésta en el fichero de configuración del framework.</p>
	 * 
	 * @param contextObject Objeto que encapsula los datos de contexto de la petición recibida.
	 * @param mapping Objeto que encapsula la configuración a seguir para procesar la petición recibida.
	 */
	public void dispatch(ContextObject contextObject, Mapping mapping)
	{
		ResourceAndData resourceAndData = null;
		FormErrors formErrors = null;
		Form form = null;
		List<Interceptor> interceptors = null;

		try
		{
			//Se comprueba que se haya podido recuperar la configuración vinculada con la petición.
			if (mapping!=null)
			{
				//Se obtienen los interceptores vinculados con la acción en curso.
				interceptors = getInterceptorList (mapping);
				
				//Se ejecuta el método doPreprocess en cada uno de los interceptores definidos para la acción en curso.
				//En el orden en el que fueron declarados en el fcihero xml de configuración del framework.
				handleInitInterceptors (contextObject, interceptors);
				
				//Se instancia la clase encargada de procesar el formulario de entrada y se carga con los
				//datos recibidos en el formulario.
				form = loadForm (contextObject, mapping);

				//Se llevan a cabo las validaciones básicas del formulario de entrada 
				//sólo si no el xml de configuración no indica lo contrario para el action en curso.
				if (mapping.getAction().isValidate())
					formErrors = handleForm (contextObject, form);
	
				//Si la validación del formulario ha detectado errores.
				if (formErrors != null && !formErrors.isEmpty())
					resourceAndData = handleErrorForm(contextObject, mapping, form, formErrors);
				else
					resourceAndData = handleAction(contextObject, mapping, form);					
				
				//El objeto resourceAndData tendrá datos si, tras procesar el Action, el resultado devuelto 
				//por éste hace referencia a un recurso definido en la declaracion del Action. 
				if (resourceAndData!=null)
				{
					//Se puebla el objeto contexto con los datos a enviar a la vista.
					populateData(contextObject, resourceAndData.getData());
				
					//Se procesan los interceptores de salida.
					handleEndInterceptors (contextObject, interceptors);
					
					if (resourceAndData.isRedirect())
						ViewUtils.redirect(contextObject, resourceAndData.getResource());
					else
						ViewUtils.forwardTo(contextObject, resourceAndData.getResource());
				}
				else //Se desconoce el destino de la petición en curso 
					ViewUtils.forwardTo(contextObject);
			}
			else //No existe un mapping en la configuración del framework para procesar la petición en curso.
				ViewUtils.forwardTo(contextObject);
		}
		catch (InterceptorException e)
		{
			log.error(e);
			//Problema al crear las instancias de los Interceptores vinculados con el Action en curso.
			ViewUtils.forwardTo (contextObject, e);
		}
		catch (FormException fe)
		{
			log.error(fe);
			//Problema al crear la instancia del formulario vinculado con el Action en curso.
			ViewUtils.forwardTo (contextObject, fe);
		}
		catch (ActionException ae)
		{
			log.error(ae);
			//Problema al crear la instancia del Action asociado a la petición en curso.
			ViewUtils.forwardTo (contextObject, ae);
		}
		catch (ViewException ve)
		{
			log.error(ve);
			//Problema al despachar la presentación de los datos en la vista destino.
			ViewUtils.forwardTo (contextObject, ve);
		}
		catch (java.lang.Exception e)
		{
			//Cualquier otra excepción capturada que no sea ninguna de las del framework
			//ha de evaluarse por si es una de las definidas por el usuario.
			try
			{
				if ((resourceAndData = handleCustomExceptions(contextObject, mapping, form, e))!=null)
				{
					populateData(contextObject, resourceAndData.getData());
					ViewUtils.forwardTo (contextObject, resourceAndData.getResource());
				}
				else
				{ 	//La excepción sucedida no coincide con ninguna de las excepciones definidas por el usuario.
					log.error(e);
					ViewUtils.forwardTo (contextObject, e);
				}
			}
			catch (Exception ve)
			{
				log.error(ve);
				ViewUtils.forwardTo (contextObject, ve);
			} 
		}
	}

	/**
	 * Método que genera una listado con las instancias de los interceptores
	 * definidos para la petición en curso.
	 * 
	 * @param mapping Mapeo vinculado a la petición en curso.
	 * @return Coleccion de interceptores vinculados con la petición en curso.
	 * @throws InterceptorException Situación de error durante la localización/instanciación
	 * de alguno de los interceptores vinculados a la petición en curso.
	 */
	private List<Interceptor> getInterceptorList(Mapping mapping) throws InterceptorException
	{
		List<Interceptor> interceptors = null;
		List<InterceptorRef> interceptorsRef = mapping.getAction().getInterceptorRef();
		if (interceptorsRef!=null && interceptorsRef.size()>0)
		{
			interceptors = new ArrayList<Interceptor>();
			for (InterceptorRef currentInterceptorRef : interceptorsRef)
				interceptors.add(
						CommandFactory.getInstance().getInterceptor(
								mapping.getInterceptors().get(currentInterceptorRef.getName())));
		}
		return interceptors;
	}

	/**
	 * Método que ejecuta el código del método doPreprocess definido en cada uno de los interceptores asociados
	 * a la petición en curso.
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param interceptorList Lista ordenada de interceptores vinculados con la petición en cusro. 
	 * @throws Exception Situación de error sucedida durante la ejecución del método doPreprocess en alguno de los interceptores.
	 */
	private void handleInitInterceptors(ContextObject contextObject, List<Interceptor> interceptorList) throws Exception 
	{
		if (interceptorList!=null)
		{
			for (Interceptor currentInterceptor : interceptorList)
				currentInterceptor.doPreprocess(contextObject);	
		}
	}
	
	/**
	 * Método que dado un form ejecuta su mnétodo validate.
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param form Formulario vinculado a la petición en curso.
	 * @return Instancia de FormErrors si se han detectado errores en la validación de los datos
	 * enviados en la petición. Instancia sin contenido o null en caso contrario.
	 * @throws java.lang.Exception
	 */
	private FormErrors handleForm (ContextObject contextObject, Form form) 
	throws java.lang.Exception
	{
		FormErrors formErrors = null;
		if (form!=null)
			formErrors = form.validate(contextObject);
		return formErrors;
	}
	
	/**
	 * Método destinado a recuperar y poblar el formulario vinculado con la petición en curso.
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param mapping Mapeo vinculado a la petición en curso.
	 * @return Instancia de la clase definida por el desarrollador que implementa la interfaz Form.
	 * @throws FormException Excepción que indica una situación de error durante la
	 * localizacion/instanciación del formulario.
	 */
	private Form loadForm (ContextObject contextObject, Mapping mapping) 
	throws FormException 
	{
		Form form = null;
		//Se comprueba que la acción a ejecutar tenga un formulario asociado. 
		if (mapping.getForm()!=null)
		{
			//Se obtiene la instancia de la clase Form.
			form =  CommandFactory.getInstance().getForm(mapping.getForm().getClazz());
			if (form!=null) //Carga la clase Form con los atributos que vienen en la petición.
				ReflectionUtilities.mapToForm(contextObject.getParameters(), form);
		}
		return form;
	}
	
	/**
	 *  Método encargado de ejecutar el código asociado a la acción en curso y generar en consecuencia
	 *  el objeto ResourceAndData que contenga tanto los datos como el destino al que enviar
	 *  el resultado de la acción.
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param mapping Mapeo vinculado a la petición en curso.
	 * @param form Formulario vinculado a la petición en curso.
	 * @return Objeto ResourceAndData que encapsula tanto los datos como el destino de la petición
	 * tras la ejeución del método execute de la Acción.
	 * @throws java.lang.Exception Excepción que indica una situación de error sucedida durante la
	 * elecución de la acción.
	 */
	private ResourceAndData handleAction(ContextObject contextObject, Mapping mapping, Form form) 
	throws java.lang.Exception 
	{
		Forward forward = null;
		ResourceAndData resourceAndData = null;
		ActionResponse actionResponse = null;
		Action action =  CommandFactory.getInstance().getAction(mapping.getAction().getClazz());
		if (action!=null)
		{
			actionResponse = action.execute(contextObject, form);
			if (actionResponse!=null)
			{
				forward = mapping.getForwards().get(actionResponse.getResource());
				if (forward!=null)
				{
					resourceAndData = new ResourceAndData(
							actionResponse.getData(),
							forward.getPath(),
							forward.isRedirect());
				}
			}
		}
		return resourceAndData;
	}
	
	/**
	 * Método encargado de generar el objeto ResourceAndData, utilizado internamente por el 
	 * framework para agrupar contenido y destino del mismo, tras la ejecución del método 
	 * validate de un formulario en el que se han detectado errores (FormError contiene elementos). 
	 *  
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param mapping Mapeo vinculado a la petición en curso.
	 * @param form Formulario vinculado a la petición en curso.
	 * @param formErrors Errores detectados en el formulario como resultado del método validate del mismo.
	 * @return Objeto ResourceAndData que encapsula tanto los datos como el destino de la petición
	 * tras la validación de un formulario que ha detectado errores.
	 */
	private ResourceAndData handleErrorForm (ContextObject contextObject, Mapping mapping, Form form, FormErrors formErrors)
	{
		ResourceAndData viewAndData = null;
		String path = mapping.getAction().getInput();
		
		//Se cargan los datos del formulario
		Map<String,Object> data = ReflectionUtilities.formToMap(form);
		
		//Si la ejecución de la accion vinculada al formulario ha generado errores de validación 
		//Se crea un atributo en la request con los mensajes de error.
		if (formErrors!=null)
			contextObject.getRequest().setAttribute(Constants.JavaMVC_ErrorMessages, formErrors);
		
		if (path!=null)
			viewAndData = new ResourceAndData(data, path, false);
		
		return viewAndData;
	}
	
	/**
	 * Método que ejecuta el código del método doPostprocess definido en cada uno de los interceptores asociados
	 * a la petición en curso.
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param interceptorList Colección que contiene las instancias de los interceptores vinculados con la petición
	 * en curso.
	 * @throws InterceptorException Excepción que indica una situación de error durante la ejecución del método doPostProcess
	 * de los interceptores definidos para la petición en curso.
	 */
	private void handleEndInterceptors(ContextObject contextObject, List<Interceptor> interceptorList) 
	throws java.lang.Exception
	{
		if (interceptorList!=null)
		{
			for (int index = interceptorList.size()-1; index>=0 ;index--)
				interceptorList.get(index).doPostprocess(contextObject);
		}
	}
	
	/**
	 *  * Método encargado de comprobar si la excepción producida coincide con alguna de las excepciones
	 * definidas por el usuario.
	 * Si la excepción no coincide directamente con ninguna de las mapeadas entonces se realiza un recorrido
	 * sobre cada una de las excepciones declaradas comprobando si la excepción sucedida es una subclase de 
	 * alguna de las excepciones declaradas.
	 * 
	 * Si se encuentra una coincidencia (directa o por herencia) se ejecuta el código de usuario definido para ella.
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param mapping Mapeo vinculado a la petición en curso.
	 * @param form Formulario vinculado a la petición en curso.
	 * @param exception
	 * @return Objeto ResourceAndData que encapsula el destino y los datos a mostrar en éste como resultado de la
	 * ejecución sucedida.
	 * @throws Exception Excepción que indica una situación de error sucedida durante la ejecución del código asociado
	 * al tratamiento de una excepción definida por el usuario.
	 */
	private ResourceAndData handleCustomExceptions(
			ContextObject contextObject, 
			Mapping mapping, 
			Form form, 
			java.lang.Exception exception)
	throws Exception
	{
		Data data = null;
		ExceptionHandler exceptionHandler = null;
		edu.uoc.framework.core.configuration.Exception configuredException = null;
		ResourceAndData resourceAndData = null;
		String ocurredException = exception.getClass().getCanonicalName();
		
		if (mapping.getExceptions()!=null)
		{
			//Se comprueba si entre los handlers declarados en el XML se encuentra declarada la excepción sucedida.
			if ((configuredException = mapping.getExceptions().get(ocurredException))==null)
			{//La excepción sucedida no está declarada como tal en el XML de configuración
		     //Se hace necesario comprobar si ésta tiene alguna excepción antecesora que sí coincida.
				String currentExceptionType = null;
				boolean found = false;
				for (Iterator<String> iterator = mapping.getExceptions().keySet().iterator(); iterator.hasNext() && !found;) 
				{
					currentExceptionType = iterator.next();
					found = ReflectionUtilities.hasAncestor(exception, currentExceptionType);
				}
				if (found)
					configuredException = mapping.getExceptions().get(currentExceptionType);
			}
			if (configuredException!=null)
			{
				exceptionHandler = CommandFactory.getInstance().getExceptionHandler(configuredException.getHandler());
				data = exceptionHandler.execute(contextObject, form, exception);
				resourceAndData = new ResourceAndData(data!=null?data.getData():null, configuredException.getJsp(), false);
			}
		}
		return resourceAndData;
	}
	
	/**
	 * <p>Método encargado de poblar la request de la petición en curso con los datos que 
	 * que desean enviarse a la vista.</p>
	 * 
	 * @param contextObject objeto que contiene el contexto de la petición en curso.
	 * @param data diccionario que contiene los datos que desean enviarse a la vista.
	 */
	private void populateData (ContextObject contextObject, Map<String,Object> data)
	{
		HttpServletRequest request = null;
		if (data!=null)
		{
			request = contextObject.getRequest();
			for (String currentKey : data.keySet())
				request.setAttribute(currentKey, data.get(currentKey));
		}
	}
}