package edu.uoc.framework.core;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.Action;
import edu.uoc.framework.exception.ActionException;
import edu.uoc.framework.exception.ExceptionHandlerException;
import edu.uoc.framework.exception.FormException;
import edu.uoc.framework.exception.InterceptorException;
import edu.uoc.framework.exceptionHandler.ExceptionHandler;
import edu.uoc.framework.form.Form;
import edu.uoc.framework.interceptor.Interceptor;

/**
 * <p>Clase que implementa los patrones factoría concreta y singleton y cuya función es la de dotar al framework de una clase única encargada
 * de la localización e instanciación de objetos declarados en el fichero de configuración del framework.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class CommandFactory 
{
	private static Logger log = Logger.getLogger(CommandFactory.class);
	private static CommandFactory instance = null;
	private static final String CLASS_NOT_FOUND = "class.not.found";
	private static final String ILLEGAL_ACCESS_EXCEPTION = "illegal.access";
	private static final String INSTANTATION_EXCEPTION = "instantation.exception";
	
	private CommandFactory(){
	}
	
	private static synchronized void createInstance()
	{
		if (instance==null)
			instance = new CommandFactory();
	}
	
	/**
	 * Método que recupera la única instancia disponible de CommandFactory.
	 * @return {@link CommandFactory}
	 */
	public static CommandFactory getInstance()
	{
		if (instance == null)
			createInstance();
		return instance;
	}
	
	public Object clone() throws CloneNotSupportedException {
    	throw new CloneNotSupportedException(); 
	}
	
	private Object getClass(String clazz)
	throws InstantiationException, IllegalAccessException, ClassNotFoundException {
		return Class.forName(clazz).newInstance();
	}
	
	/**
	 * Método que crea una instancia de la clase Interceptor especificada.
	 * 
	 * @param clazz nombre de la clase de que se quiere instanciar.
	 * @return Instancia de {@link edu.uoc.framework.interceptor.Interceptor} especificada por clazz.
	 * @throws InterceptorException Excepción que encapsula los posibles problemas sucedidos durante
	 * la instanciación de una clase de tipo Interceptor.
	 */
	public Interceptor getInterceptor(String clazz) 
	throws InterceptorException 
	{
		try
		{
			return (Interceptor)getClass(clazz);
		}
		catch (ClassNotFoundException cnfe)
		{
			log.error(MessageResourceBundle.getMessage(CLASS_NOT_FOUND, clazz));
			throw new InterceptorException(cnfe);
		}
		catch (IllegalAccessException iae)
		{
			log.error(MessageResourceBundle.getMessage(ILLEGAL_ACCESS_EXCEPTION, clazz));
			throw new InterceptorException(iae);
		}
		catch (InstantiationException e1)
		{
			log.error(MessageResourceBundle.getMessage(INSTANTATION_EXCEPTION, clazz));
			throw new InterceptorException(e1);
		}
	}
	
	/**
	 * Método que crea una instancia de la clase action especificada.
	 * 
	 * @param clazz nombre de la clase de tipo Action que se quiere instanciar.
	 * @return Instancia de {@link edu.uoc.framework.action.Action} especificada por clazz.
	 * @throws ActionException Excepción que encapsula los posibles problemas sucedidos durante
	 * la instanciación de una clase de tipo Action.
	 */
	public Action getAction(String clazz)
	throws ActionException 
	{
		try
		{
			return (Action)getClass(clazz);
		}
		catch (ClassNotFoundException cnfe)
		{
			log.error(MessageResourceBundle.getMessage(CLASS_NOT_FOUND, clazz));
			throw new ActionException(cnfe);
		}
		catch (IllegalAccessException iae)
		{
			log.error(MessageResourceBundle.getMessage(ILLEGAL_ACCESS_EXCEPTION, clazz));
			throw new ActionException(iae);
		}
		catch (InstantiationException e1)
		{
			log.error(MessageResourceBundle.getMessage(INSTANTATION_EXCEPTION, clazz));
			throw new ActionException(e1);
		}
	}
	
	/**
	 * Método que crea una instancia de la clase Form especificada.
	 * 
	 * @param clazz pakage y nombre de la clase de tipo Form que se quiere instanciar.
	 * @return Instancia de {@link edu.uoc.framework.form.Form} especificada por clazz.
	 * @throws FormException Excepción que encapsula los posibles problemas sucedidos durante
	 * la instanciación de una clase de tipo Form.
	 */
	public Form getForm(String clazz)
	throws FormException
	{
		try
		{
			return (Form)getClass(clazz);
		}
		catch (ClassNotFoundException cnfe)
		{
			log.error(MessageResourceBundle.getMessage(CLASS_NOT_FOUND, clazz));
			throw new FormException(cnfe);
		}
		catch (IllegalAccessException iae)
		{
			log.error(MessageResourceBundle.getMessage(ILLEGAL_ACCESS_EXCEPTION, clazz));
			throw new FormException(iae);
		}
		catch (InstantiationException e1)
		{
			log.error(MessageResourceBundle.getMessage(INSTANTATION_EXCEPTION, clazz));
			throw new FormException(e1);
		}
	}
	
	/**
	 * Método que crea una instancia de la clase ExceptionHandler especificada.
	 * 
	 * @param clazz pakage y nombre de la clase de tipo ExceptionHandler que se quiere instanciar.
	 * @return Instancia de {@link edu.uoc.framework.exceptionHandler.ExceptionHandler} especificada por clazz.
	 * @throws ExceptionHandlerException Excepción que encapsula los posibles problemas sucedidos durante
	 * la instanciación de una clase de tipo ExceptionHandler.
	 */
	public ExceptionHandler getExceptionHandler(String clazz)
	throws ExceptionHandlerException
	{
		try
		{
			return (ExceptionHandler)getClass(clazz);
		}
		catch (ClassNotFoundException cnfe)
		{
			log.error(MessageResourceBundle.getMessage(CLASS_NOT_FOUND, clazz));
			throw new ExceptionHandlerException(cnfe);
		}
		catch (IllegalAccessException iae)
		{
			log.error(MessageResourceBundle.getMessage(ILLEGAL_ACCESS_EXCEPTION, clazz));
			throw new ExceptionHandlerException(iae);
		}
		catch (InstantiationException e1)
		{
			log.error(MessageResourceBundle.getMessage(INSTANTATION_EXCEPTION, clazz));
			throw new ExceptionHandlerException(e1);
		}
	}
}
