package edu.uoc.framework.core;


import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.xml.XMLConstants;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import edu.uoc.framework.core.configuration.Action;
import edu.uoc.framework.core.configuration.Exception;
import edu.uoc.framework.core.configuration.Form;
import edu.uoc.framework.core.configuration.Forward;
import edu.uoc.framework.core.configuration.FrameworkConfig;
import edu.uoc.framework.core.configuration.InterceptorRef;
import edu.uoc.framework.exception.ConfigurationFileException;

/**
 * <p>Clase de apoyo al framework en la que se delega la lectura y validación del fichero de configuración del framework.</p>
 * 
 * <p>Proporciona un constructor que establece el fichero de configuración y una serie de métodos que permiten tanto la validación como la recuperación
 * de la configuración contenida en este.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class Configuration 
{
	private static Logger log = Logger.getLogger(Configuration.class);
	
	private static final String FRAMEWORK_CONFIGURATION_FILE_ACCESS_ERROR = "framework.configuration.file.access.error";
	private static final String FRAMEWORK_CONFIGURATION_FILE_ACCESS_OK = "framework.configuration.file.access.ok";
	private static final String FRAMEWORK_CONFIGURATION_FILE_PARSE_ERROR ="framework.configuration.file.parse.error";
	private static final String FRAMEWORK_CONFIGURATION_FILE_PARSE_OK ="framework.configuration.file.parse.ok";
	private static final String FRAMEWORK_CONFIGURATION_FILE_VALIDATION_ERROR = "framework.configuration.file.validation.error";
	private static final String FRAMEWORK_CONFIGURATION_FILE_VALIDATION_OK = "framework.configuration.file.validation.ok";
	
	/**
	 * Propiedad que recoge la configuración del framework leída del fichero de configuración.
	 */
	private FrameworkConfig frameworkConfig = null;

	private String configurationFile = null;
	
	/**
	 * <p>Constructor que crea una instancia de Configuration vinculada con el fichero
	 * de configuración indicado en el parámetro javaMVCconfigurationFile.</p>
	 *  
	 * @param javaMVCconfigurationFile Path con la ubicación del fichero de configuración del framework.
	 */
	public Configuration(String javaMVCconfigurationFile){
		this.configurationFile = javaMVCconfigurationFile;
	}
	
	@SuppressWarnings("unused")
	private Configuration(){
	}
	
	/**
	 * <p>Método encargado de comprobar la disponibilidad del fichero de configuración,
	 * validar que esté bien formado según el schema 'JavaMVC.xsd' y finalmente
	 * encargado de extraer su información y almacenarla en la propiedad frameworkConfig.</p>
	 * 
	 * @throws ConfigurationFileException - Excepción que indicará que algún inconveniente 
	 * en el fichero de configuración ha impedido cargar la configuración del framework correctamente.
	 */
	public void loadConfiguration() 
	throws ConfigurationFileException
	{
		String message = null;
		File file = null;
		if (configurationFile != null)
		{
			file = new File (configurationFile);
			//Se comprueba que el fichero de configuración proporcionado exista y tenga permisos de lectura.
			if (file.exists() && file.canRead())
			{
				if (log.isDebugEnabled())
					log.debug(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_ACCESS_OK));
				
				//Se valida que el XML sea correcto según el schema JavaMVC.xsd.
				if (validateXmlFile(file))
					parseXmlFile(file); //Se procesa el XML validado extrayendo la información que éste contiene.
				else
				{ //El fichero XML de configuración NO cumple con el schema JavaMVC.xsd
					message = MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_VALIDATION_ERROR);
					log.error(message);
					throw new ConfigurationFileException(message);
				}
			}
			else //El fichero no se encuentra en la ruta especificada o no tiene permisos de lectura.
			{
				message = MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_ACCESS_ERROR);
				log.error(message);
				throw new ConfigurationFileException(message);
			}
		}
		else //No se ha proporcionado una ruta de la que leer el fichero de configuración
		{
			message = MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_ACCESS_ERROR);
			log.error(message);
			throw new ConfigurationFileException(message);
		}
	}
	
	/**
	 * <p>Método que comprueba si el fichero xmlFile cumple con el schema definido por JavaMVC.xsd.</p>
	 * 
	 * @param xmlFile {@ java.io.File} que apunta al fichero físico que contiene el XML a validar.
	 * @return true si el fichero XML cumple el Schema definido por JavaMVC.xsd o false en caso contrario.
	 * @throws ConfigurationFileException - Excepción que se lanza cuando sucede un error
	 * durante el proceso de validación del XML. 
	 */
	private boolean validateXmlFile(File xmlFile) 
	throws ConfigurationFileException 
	{
		boolean validXmlFile = false;
		SchemaFactory factoryXmlSchema = null;
		Schema schemaXml = null;
		Validator validator = null;
		URL urlSchema = null;
        try 
        {
        	urlSchema = Configuration.class.getResource("/JavaMVC.xsd");
        	factoryXmlSchema = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
		    schemaXml = factoryXmlSchema.newSchema(urlSchema);
		    validator = schemaXml.newValidator();
		    validator.validate(new StreamSource(xmlFile));
		    
		    if (log.isDebugEnabled())
		    	log.debug(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_VALIDATION_OK));
		    
		    validXmlFile = true;
        }
        catch (SAXException saxe)
        {
        	log.error(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_VALIDATION_ERROR));
        	throw new ConfigurationFileException(saxe);
        }
        catch (IOException ioe)
        {
        	log.error(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_VALIDATION_ERROR));
        	throw new ConfigurationFileException(ioe);
        }
		return validXmlFile;
	}
	
	/**
	 * <p>Método que a partir de un fichero XML que sigue el schema JavaMVC.xsd y
	 * una colección de clases apropiadamente anotadas que representan los datos 
	 * que puede contener el XML, utiliza JAXB para poblar estas clases a partir
	 * del contenido del XML.</p>
	 * 
	 * @param xmlFile {@link java.io.File} de la configuración del framework.
	 * @return true si se ha podido extraer toda la información del fichero. false en caso contrario.
	 * @throws ConfigurationFileException - Excepción que se lanza cuando el proceso de extracción de
	 * datos del fichero 'xmlFile' no ha podido finalizar satisfactoriamente.
	 */
	private boolean parseXmlFile(File xmlFile) 
	throws ConfigurationFileException
	{
		JAXBContext jaxbContext = null;
		Unmarshaller jaxbUnmarshaller = null;
		boolean xmlFileProcessed = false;
		try
		{
			/*Se le proporciona a JAXB el package que contiene las clases que se utilizarán
			* a lo largo del proceso de unmarshaling del XML.*/
			jaxbContext = JAXBContext.newInstance("edu.uoc.framework.core.configuration");
			jaxbUnmarshaller = jaxbContext.createUnmarshaller();
			this.frameworkConfig = (FrameworkConfig) jaxbUnmarshaller.unmarshal(xmlFile);
			
			if (log.isDebugEnabled())
				log.debug(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_PARSE_OK));
			
			xmlFileProcessed = true;
		}
		catch (JAXBException jaxbe)
		{ //Se ha producido un error durante el proceso de unmarshaling del XML.
			log.error(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_PARSE_ERROR));
			throw new ConfigurationFileException(jaxbe);
		}
		return xmlFileProcessed;
	}
	
	/**
	 * 
	 * Método que permite consultar el valor de la propiedad frameworkConfig.
	 * 
	 * @return Instancia de {@link FrameworkConfig}
	 */
	public FrameworkConfig getFrameworkConfig(){
		return frameworkConfig;
	}

	/**
	 * Método que recupera de la configuración del framework, la parte perteneciente a un Action concreto.
	 * 
	 * @param actionUrl Nombre del Action del que se quiere recuperar la configuración.
	 * @return objeto Mapping que contiene la configuración asociada al Action.
	 */
	public Mapping getMapping(String actionUrl)
	{
		Mapping mapping = null;
		Action action = null;
		Form form = null;
		String interceptor = null;
		
		Map<String,String> interceptors = null;
		Map<String,Exception> exceptions = null;
		Map<String,Forward> forwards = null;
		
		//Sólo si ya se ha cargado la configuración del framework. (Ello implica que el XML se ha localizado, leído, validado y procesado).
		if (frameworkConfig!=null && frameworkConfig.getActions().containsKey(actionUrl))
		{
			//Se recupera la configuración del Action.
			action = frameworkConfig.getActions().get(actionUrl);
			
			//Se recuperan las excepciones globales que deberám tenerse en cuenta para cualquier Action. 
			exceptions = frameworkConfig.getExceptions();
			
			//Si el nombre del formulario que usa el action no se corresponde al 100% con el nombre
			//de un form previamente declarado, se considerará que el action no tiene formulario asociado.
			if (action.getActionForm()!=null)
				form = frameworkConfig.getForms().get(action.getActionForm());
			
			//Si el action tiene referencias a interceptores declaradas se cargará la definición de cada uno de ellos.
			if (!action.getInterceptorRef().isEmpty())
			{
				interceptors = new HashMap<String, String>();
				for (InterceptorRef currentInterceptorRef : action.getInterceptorRef())	{
					interceptor = frameworkConfig.getInterceptors().get(currentInterceptorRef.getName());
					interceptors.put(currentInterceptorRef.getName(), interceptor);
				}
			}
			
			//Si el action tiene forwards a otros recursos (JSP's, Action's) declarados.
			if (!action.getForward().isEmpty())
			{
				forwards = new HashMap<String,Forward>();
				for (Forward currentForward : action.getForward())
					forwards.put(currentForward.getName(),currentForward);
			}

			mapping = new Mapping(action, form, exceptions, interceptors, forwards);
		}
		return mapping;
	}
}