package edu.uoc.framework.core;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * <p>Clase que encapsula el concepto de contexto de una petición.</p>
 *
 * <p>Dado que las peticiones recibidas por el framework son
 * peticiones HTTP que atrapará el {@link edu.uoc.framework.core.FrontController}
 * (tecnología Servlet), toda petición tendrá asociados un objeto
 * HttpServletRequest y otro HttpServletResponse que encapsularán los datos
 * generados durante el envío de la petición HTTP al cliente y durante la 
 * respuesta HTTP que el servidor le enviará al cliente.</p>
 *  
 * @author Alejandro Marmelo Insua
 */
public class ContextObject {
	
	/**
	 * Objeto que encapsula los datos generados durante el envío de la petición HTTP al cliente.
	 */
	private HttpServletRequest request;
	
	/**
	 * Objeto que encapsula la respuesta HTTP que el servidor enviará al cliente.
	 */
	private HttpServletResponse response;
	
	/**
	 * Conjunto de parámetros recibidos con la petición.
	 */
	private Map<String,String[]> parameters;
	
	public ContextObject(HttpServletRequest request,
			HttpServletResponse response,
			Map<String,String[]> parameters)
	{
		this.request = request;
		this.response = response;
		this.parameters = parameters;
	}
	
	/**
	 * Método que recupera el objeto HttpServletRequest asociado a la petición.
	 * 
	 * @return {@link HttpServletRequest}
	 */
	public HttpServletRequest getRequest() {
		return request;
	}

	/**
	 * Método que permite establecer el objeto HttpServletRequest asociado con una petición.
	 * 
	 * @param request Objeto HttpServletRequest de la petición.
	 */
	public void setRequest(HttpServletRequest request) {
		this.request = request;
	}

	/**
	 * Método que permite recuperar el objeto HttpServletResponse asociado con una petición. 
	 * 
	 * @return {@link HttpServletResponse}
	 */
	public HttpServletResponse getResponse() {
		return response;
	}

	/**
	 * Método que permite establecer el objeto HttpServletResponse asociado con la petición.
	 * 
	 * @param response {@link HttpServletResponse}
	 */
	public void setResponse(HttpServletResponse response) {
		this.response = response;
	}

	/**
	 * Método que permite recuperar los parámetros recibidos con la petición.
	 * 
	 * @return Colección de pares <nombre,valores> que representan los datos capturados en la petición recibida.
	 */
	public Map<String, String[]> getParameters() {
		return parameters;
	}

	/**
	 * Método que establece el conjunto de parámetros recibidos con una petición.
	 * @param parameters Colección de pares <nombre,valores> que representa la información recibida con una petición.
	 */
	public void setParameters(Map<String, String[]> parameters) {
		this.parameters = parameters;
	}
}