package edu.uoc.framework.core;


import java.io.IOException;
import java.io.UnsupportedEncodingException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import edu.uoc.framework.exception.ConfigurationFileException;
import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.StringUtils;

/**
 * <p>Clase que implementa el patrón FrontController y será por tanto 
 * la encargada de proporcionar un único punto de acceso a la capa de presentación.</p>
 * 
 * <p>Dado que esta clase es un Servlet será necesario declararlo
 * en el descriptor de despliegue de la aplicación Web (fichero web.xml)
 * asignándole un patrón de URLs de forma que todas aquellas peticiones
 * recibidas que sigan el patrón, sean procesadas por éste.</p>
 * 
 * <pre>
 * {@code
 *<servlet-mapping>
 *	<servlet-name>FrontController</servlet-name>
 *	<url-pattern>*.do</url-pattern>
 *</servlet-mapping>}
 * </pre>
 *  
 * <p>Para cada petición que cumpla el patrón <url-pattern> definido en 
 * el fichero web.xml, ésta clase se encargará de generar un  
 * {@link edu.uoc.framework.core.ContextObject} con la información del contexto de la petición,
 * así como un objeto {@link edu.uoc.framework.core.Mapping} que contendrá los datos del fichero de configuración del framework 
 * asociados con la petición en curso. Finalmente se despachará esta información a la clase {@link ApplicationController} de forma
 * que la gestión de invocación a las acciones y la gestión de envío a las vistas quede fuera del FrontController.</p>
 * 
 * <p>Este servlet, como cualquier otro, debe estar declarado en el 
 * fichero de despliegue de la aplicación web 'web.xml'.</p>
 * <p>Puesto que será el encargado de 'inicializar' el framework antes
 * de que éste sirva la primera petición, será necesario indicarle el 
 * fichero de configuración que utilizará durante su ejecución.</p>
 * 
 * <p>La ruta a este fichero debe indicársele como valor del tag {code <param-value> }
 * para el parámetro con nombre 'config' ( {@code <param-name>config</param-name>} ) tal y 
 * como se muestra en el siguiente ejemplo.</p>
 * <pre>
 * {@code
 *<servlet>
 *	<display-name>FrontController</display-name>
 *	<servlet-name>FrontController</servlet-name>
 *	<servlet-class>edu.uoc.framework.core.FrontController</servlet-class>
 *	<init-param>
 *		<param-name>config</param-name>
 *		<param-value>/WEB-INF/classes/resources/JavaMVC.xml</param-value>
 *	</init-param>
 *	<load-on-startup>1</load-on-startup>
 *</servlet>
 * }
 * </pre>
 * 
 * <p>El fichero JavaMVC.xml al que se hace referencia en el ejemplo,
 * debe ser un fichero XML que cumpla el esquema JavaMVC.xsd ya que, en caso contrario,
 * el framework no será capaz de inicializarse correctamente.</p>
 * 
 */
public class FrontController extends HttpServlet 
{
	private static final long serialVersionUID = 3453474517546044555L;
	
	private static Logger log = Logger.getLogger(FrontController.class);
		
	private static final String FRAMEWORK_CONFIGURATION_FILE_NOT_PROVIDED = "framework.configuration.file.not.provided";
	private static final String FRAMEWORK_CONFIGURATION_FILE_PATH_NOT_AVAILABLE = "framework.configuration.file.path.not.available";
	private static final String FRAMEWORK_CONFIGURATION_FILE_LOADED = "framework.configuration.file.loaded";
	private static final String FRAMEWORK_CONFIGURATION_INVALID_URL_PATTERN = "framework.configuration.invalid.url.pattern";
	private static final String CURRENT_ACTION_REQUEST = "current.action.request";
	private static final String INVALID_ENCODING_DEFINED = "invalid.encoding.defined";
	
	/**
	 * Configuración de la aplicación.
	 */
	private Configuration configuration = null;
	
	/**
	 * Propiedad que almacena el encoding definido por el desarrollador
	 * y que deberá aplicarse a toda petición recibida por el frontController.
	 */
	private String encoding = null;

	/**
	 * <p>Método de inicialización invocado por el servidor que se ejecutará una única vez
	 * justo después de ser instanciado.</p>
	 * 
	 * En función del servidor en el que se despliegue la aplicación este proceso puede
	 * ocurrir en distintos momentos:
	 *  <ol><li>- Cuando arranca el servidor.</li>
	 *  <li>- La primera vez que se invoca al servlet, justo antes de llamar el método service().</li>
	 *  <li>- Etc..</li>
	 *  </ol>
	 *  
	 *  <p>En cualquier caso este método se habrá ejecutado antes de que se procese la primera
	 *  petición.</p>
	 *
	 * <p>Dadas las cualidades de este método será el apropiado para llevar a cabo las tareas
	 * de inicialización del framework.</p>
	 * 
	 * <p>Es necesario que en la declaración del Servlet (web.xml) se informe un parámetro de inicio
	 * cuyo valor sea la ruta al fichero xml de configuración del framework (que deberá cumplir 
	 * con el esquema JavaMVC.xsd).</p>
	 * 
	 * @param servletConfig - Objeto usado por el contenedor de Servlets utilizado para pasar
	 * información a un servlet durante su inicialización.
	 * @throws ServletException - Excepción lanzada por el servlet ante una situación de error.
	 */
	public void init(ServletConfig servletConfig) throws ServletException
	{
		String config = null; 
		String configFile = null;
		try
		{
			super.init(servletConfig);
			
			//Se establece el encoding con el que se recuperarán los datos de una petición.
			this.encoding = servletConfig.getInitParameter("encoding");
			
			//Se recupera el valor del parámetro config definido como parámetro de inicio del servlet FrontController.
			if ((config = servletConfig.getInitParameter("config"))!=null)
			{
				configFile = servletConfig.getServletContext().getRealPath(config);
				if (configFile!=null)
				{
					//Se crea la clase COnfiguration con el fichero de configuración del Framework
					configuration = new Configuration(configFile);
					//Se le pide a Configuration que lleve a cabo la carga del fichero (lectura, validación y recuperación de datos).
					configuration.loadConfiguration();
					
					if (log.isInfoEnabled())
						log.info(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_LOADED, config));
					
					//Se carga en el contexto de la aplicación aquella información relevante para el framework que aplicará a toda petición.
					configureApplicationContextScope(servletConfig);
				}
				else //No se consigue encontrar el fichero en la ruta indicada.  
					throw new ServletException(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_PATH_NOT_AVAILABLE, config));
			}
			else //No se ha especificado en el descriptor de despliegue ("web.xml") la ruta del fichero de configuración del framework. 
				throw new ServletException(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_FILE_NOT_PROVIDED)); 
		}
		catch (ConfigurationFileException e)
		{ //Se ha producido alguna excepción durante el proceso de lectura del fichero de configuración del framework. 
			log.error(e);
			throw new ServletException(e);
		}
	}
	
	/**
	 * <p>Método encargado de configurar en el contexto de la aplicación web (único 
	 * por aplicación web y máquina virtual) aquellas constantes necesarias 
	 * para el framework durante su ejecución y que aplicarán por igual a todas 
	 * las peticiones.</p>
	 * 
	 * <p>Estas constantes son:</p> 
	 * 			<ul><li>Bundle que se utilizará para la internacionalización de 
	 * 			la aplicación y que se definirá en el XML de configuración del framework.</li>
	 * 
	 * 			<li>El patrón de URLs que serán servidas por el FrontController y que se encuentra
	 * 			configurado en el descriptor de despliegue de la aplicación web 'web.xml'.</li>
	 * 
	 * @param servletConfig - Objeto usado por el contenedor de Servlets utilizado para pasar información 
	 * 							a un servlet durante su inicialización.
	 * @throws ServletException - Excepción lanzada por el servlet ante una situación de error.
	 */
	private void configureApplicationContextScope(ServletConfig servletConfig) 
	throws ServletException
	{
		String i18n = null;
		Object[] mappings = null;
		
		//Se comprueba si se definió un bundle de internacionalización para la aplicación.
		//Y si fue así define en el contexto de la aplicación web un atributo con el bundle.
		if (configuration.getFrameworkConfig().getI18N()!=null)
		{
			if ((i18n = configuration.getFrameworkConfig().getI18N().getParameter())!=null);
				servletConfig.getServletContext().setAttribute(Constants.JavaMVC_I18N, i18n);
		}

		//Se recupera el patrón de URLs que serán tratadas por el FrontController.
		mappings = (Object[])servletConfig.getServletContext().getServletRegistration("FrontController").getMappings().toArray();
		
		//Se comprueba que no se haya definido más de un URL-Pattern dado que aunque la especificación Servlet lo permite
		//JavaMVC no podrá operar con más de uno.
		if (mappings!=null && mappings.length>1)
			throw new ServletException(MessageResourceBundle.getMessage(FRAMEWORK_CONFIGURATION_INVALID_URL_PATTERN, mappings));
		
		//Patrón de Urls que serán atrapadas por el FrontController. 
		servletConfig.getServletContext().setAttribute(Constants.JavaMVC_UrlPattern, mappings[0]);
	}
	
	//Método que atrapa las peticiones por GET.
	protected void doGet(
			HttpServletRequest request,
			HttpServletResponse response) 
	throws ServletException, IOException {
		processRequest(request, response);
	}

	//Método que atrapa las peticiones por POST.
	protected void doPost(
			HttpServletRequest request, 
			HttpServletResponse response) 
	throws ServletException, IOException {
		processRequest(request, response);
	}

	/**
	 *  <p>Método encargado de:</p>
	 * 	<ul><li>Recuperar los datos de cada petición generando un objeto ContextObject asociado a cada una.</li>
	 * 	<li>Extraer de la URL atrapada por el FrontController el Action solicitado.</li>
	 * 	<li>Delegar el procesamiento de la petición en el ApplicationController.</li></ul>
	 * 
	 * @param request Objeto que encapsula los datos generados durante la petición HTTP de un cliente. 
	 * @param response Objeto que ayuda al servlet con el envío de la respuesta al cliente como resultado de su petición HTTP. 
	 */
	private void processRequest(HttpServletRequest request, 
			HttpServletResponse response)
	{
		//Se comprueba y setea si es necesario el encoding de la request
		//antes de cualquier acción que acceda a los datos.
		checkAndSetEncoding(request);
		
		//Se crea el objeto Contexto
		ContextObject contextObject = new ContextObject(request, response, request.getParameterMap());
		//Se recupera el nombre de la acción que deberá ejecutarse a partir de la URL recibida.
		String currentAction = (processPath(request));
		if (log.isDebugEnabled())
			log.debug(MessageResourceBundle.getMessage(CURRENT_ACTION_REQUEST, currentAction));
		
		//Se recupera la instancia de ApplicationController y se invoca al método dispatch.
		ApplicationController.getInstance().dispatch(contextObject, configuration.getMapping(currentAction));
	}
	
	/**
	 * Método que extrae una acción de la URL recibida por el FrontController.
	 * 
	 * Los mapeos en Java pueden ser mapeos exactos, por extensión o por path.
	 * A la hora de especificar en el descriptor de despliegue de la aplicación
	 * Web qué tipo de URLs serán capturadas por el FrontController pueden aparecer
	 * situaciones como las siguiente:
	 * 
	 * <url-pattern>*.xx</url-pattern> --> Mapeo por extensión
	 * Siempre empieza por * NUNCA empezará por /
	 * Seguido del * debe haber una extensión con punto (.do, .jsp, .xxx)
	 * 
	 * ej: http://localhost:8080/appContext/xxx/yyy/zzz/action.xx?name=aaa&password=bbbb
	 * ServletPath = 
	 * PathInfo = xxx/yyy/zzz/action.xx
	 * QueryString = name=aaa&password=bbbb
	 * 
	 * <url-pattern>/xxxx/*</url-pattern> --> Mapeo por path
	 * Siempre empieza por /
	 * Siempre acaba por /*
	 * 
	 * ej: http://localhost:8080/appContext/xxx/yyy/zzz/action?name=aaa&password=bbbb
	 * ServletPath = xxxx
	 * PathInfo = /yyy/zzz/action.do
	 * QueryString = name=aaa&password=bbbb
	 *  
	 * <url-pattern>/xxxx/yyyy</url-pattern> --> Mapeo exacto
	 * Siempre empezará por /
	 * Le seguirá lo que sea incluso podrá tener extensión aunque no es obligatorio.
	 * 
	 * ej: http://localhost:8080/appContext/xxx/yyy?name=aaa&password=bbbb
	 * ServletPath = xxx/yyy
	 * PathInfo = 
	 * QueryString = name=aaa&password=bbbb
	 * 
	 * JavaMVC ha de configurase atrapando (de la forma que sea), aquellas peticiones 
	 * recibidas que NO correspondan con recursos estáticos (css,jsp,jpg,etc..) ya que esos
	 * deben ser proporcionados por el servlet por defecto que proporcione el servidor.
	 * 
	 * @param request Objeto que encapsula los datos generados durante la petición HTTP de un cliente.
	 * @return nombre del Action contenida en la petición HTTP. 
	 */
	private String processPath (HttpServletRequest request)
	{
		if (log.isDebugEnabled())
		{
			log.debug(" RequestURI [" +request.getRequestURI() + "] \r\n"
					+ " ContextPath [" + request.getContextPath() + "]\r\n"
					+ " ServletPath [" + request.getServletPath() + "]\r\n"
					+ " PathInfo [" + request.getPathInfo() + "]\r\n"
					+ " QueryString ["+ request.getQueryString() + "]\r\n");
		}
		
		String path = request.getPathInfo();
		if (StringUtils.isBlank(path))
			path = request.getServletPath();
		if (!StringUtils.isBlank(path))
		{
			int slash = path.lastIndexOf("/");
			int period = path.lastIndexOf(".");
			if (period == -1) period = path.length();
			if ((period >= 0) && (period > slash)) 
				path = path.substring(slash+1, period);
		}
		return path;
	}
	
	/**
	 * <p>Método que comprueba si la request recibida especifica o no un encoding
	 * con el que tratar los datos recibidos.</p>
	 * 
	 * <p>Si la request no define un encoding, el servidor aplicará el encoding
	 * que tenga definido por defecto a no ser el que el desarrollador
	 * haya definido uno durante el proceso de inicilialización del FrontController.</p>
	 * 
	 * <p>Aplicar el encoding correcto es vital para garantizar que los datos 
	 * lleguen al resto de componentes de la aplicación correctamente interpretados.</p> 
	 *
	 * @param request Objeto {@link HttpServletRequest}
	 */
	private void checkAndSetEncoding(HttpServletRequest request)
	{
		//Si la petición recibida por el FrontController no tiene informado
		//el encoding utilizado para enviar los datos. 
		if (StringUtils.isBlank(request.getCharacterEncoding()))
		{
			//Se comprueba si existe un encoding definido por el usuario.
			if (!StringUtils.isBlank(encoding))
			{
				try	{
					request.setCharacterEncoding(encoding);
				}
				catch (UnsupportedEncodingException uee){
					log.error(MessageResourceBundle.getMessage(INVALID_ENCODING_DEFINED, encoding));
				}
			}
			//Si la petición no tiene un encoding informado y tampoco
			//ha definido uno el usuario, se aplicará el encoding por defecto
			//del contenedor en el que se haya desplegado la aplicación.
		}
	}
}
