package edu.uoc.framework.core;

import java.util.Map;

import edu.uoc.framework.core.configuration.Action;
import edu.uoc.framework.core.configuration.Exception;
import edu.uoc.framework.core.configuration.Form;
import edu.uoc.framework.core.configuration.Forward;

/**
 * <p>Clase que representa la parte de la configuración del framework vinculada a un Action concreto.</p>
 * 
 * <p>Representa únicamente aquella porción de la configuración con la que 
 * el framework es capaz de servir un {@link edu.uoc.framework.action.Action} en concreto sin conocer más detalles
 * sobre el resto de la configuración.</p>
 * 
 * @author Alejandro Marmelo Insua
 */
public class Mapping {

	/**
	 * Propiedad que encapsula las propiedades de un Action leídas de la configuración del framework. 
	 */
	private Action action;
	
	/**
	 * Colección de Excepciones que deben tratarse vinculadas a un Action. 
	 */
	private Map<String, Exception> exceptions;
	/**
	 * Colección de Interceptores vinculados a un Action. 
	 */
    private Map<String, String> interceptors;
    /**
	 * Colección de Forwards vinculados a un Action. 
	 */
    private Map<String,Forward> forwards;
    /**
   	 * Propiedad que encapsula las propiedades de un Form leídas de la configuración del framework.  
   	 */
    private Form form;
	
    /**
     * Método que construye un Mapping con la configuración recuperada del fichero del configuración del framework
     * asociada a un Action concreto.
     * 
     * @param action Datos del {@link edu.uoc.framework.core.configuration.Action} extraídos del fichero de configuración del framework.
     * @param form Datos del {@link edu.uoc.framework.core.configuration.Form} extraídos del fichero de configuración del framework.
     * @param exceptions Diccionario de de pares (nombre,{@link edu.uoc.framework.core.configuration.Exception}) con las excepciones leídas del fichero de configuración del framework. 
     * @param interceptors Diccionario de pares (nombre, clase) con los interceptores vinculados al action leídos del fichero de configuración del framework.
     * @param forwards Diccionario de de pares (nombre,{@link edu.uoc.framework.core.configuration.Forward}) con los forwards leídos del fichero de configuración del framework.
     */
	public Mapping (
			Action action, 
			Form form, 
			Map<String, Exception> exceptions,
			Map<String,String> interceptors,
			Map<String,Forward> forwards)
	{
		this.action = action;
		this.form = form;
		this.exceptions = exceptions;
		this.interceptors = interceptors;
		this.forwards = forwards;
	}

	/**
	 * Método que recupera los datos del Action extraídos del fichero de configuración del framework.
	 * @return {@link edu.uoc.framework.core.configuration.Action}
	 */
	public Action getAction() {
		return action;
	}

	/**
	 * Método que recupera el diccionario de pares (nombre,{@link edu.uoc.framework.core.configuration.Exception}) recuperadas del fichero de configuración. 
	 * @return Diccionario de de pares (nombre,{@link edu.uoc.framework.core.configuration.Exception}) con las excepciones leídas del fichero de configuración del framework.
	 */
	public Map<String, Exception> getExceptions() {
		return exceptions;
	}

	/**
	 * Método que recupera el diccionario de pares (nombre, clase) de interceptores vinculados al Action y definidos en el fichero de configuración del framework.
	 * 
	 * @return Diccionario de pares (nombre,clase) con los interceptores definidos para el Action y definidos en el fichero de configuración del framework.
	 */
	public Map<String, String> getInterceptors() {
		return interceptors;
	}

	/**
	 * Método que recupera el Form extraído del fichero de configuración del framework.
	 * 
	 * @return {@link edu.uoc.framework.core.configuration.Form}
	 */
	public Form getForm() {
		return form;
	}

	/**
	 * Método que recupera los forwards extraídos del fichero de configuración del framework.
	 * 
	 * @return Diccionario de pares (nombre,{@link edu.uoc.framework.core.configuration.Forward}) con los forwards definidos para el Action.
	 */
	public Map<String,Forward> getForwards() {
		return forwards;
	}
}