package edu.uoc.framework.form;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import edu.uoc.framework.utilities.I18N.Message;

/**
 * <p>Clase que encapsula un conjunto de mensajes de error resultado de la validación
 * de un {@link Form}.</p>
 * 
 *  <p>Una instancia de esta clase se envía como respuesta
 *  no satisfactoria a la validación de un {@link Form} si el método 
 *  validate ha encontrado errores en los datos recibidos.</p> 
 *
 *  <p>El uso de etiquetas como ErrorTag o Message desde la vista, 
 *  permitirán recuperar y traducir estos mensajes de error
 *  mostrándolos según el locale del usuario.</p> 
 *     
 * @author Alejandro Marmelo Insua
 *
 */
public class FormErrors
{
	/**
	 * Diccionario que guarda pares <nombre, {@link edu.uoc.framework.utilities.I18N.Message}
	 */
	private Map<String, Message> errors = null;

	/**
	 * Constructor que crea una instancia de FormErrors 
	 * partiendo del nombre de un campo y su {@link edu.uoc.framework.utilities.I18N.Message} asociado.
	 * 
	 * @param key Identificador al que se asignará el Message.
	 * @param value {@link edu.uoc.framework.utilities.I18N.Message} asociado a 'key'.
	 */
	public FormErrors(String key, Message value) {
		errors = new HashMap<String, Message>();
		errors.put(key, value);
	}
	
	/**
	 * Constructor que crea una instancia de FormErrors sin mensajes de error.
	 */
	public FormErrors() {
		errors = new HashMap<String, Message>();
	}
	
	/**
	 * Método que añade a la instancia de FormErrors un {@link Message} identificado por 'key'.
	 * 
	 * @param key Identificador al que se asignará el Message.
	 * @param value {@link edu.uoc.framework.utilities.I18N.Message} asociado a 'key'.
	 */
	public void addError(String key, Message value) {
		errors.put(key, value);
	}
	
	/**
	 * Método que recupera el Message asociado a una clave.
	 * 
	 * @param key Identificador del Message que se quiere recuperar.
	 * @return {@link Message} si 'key' existe o null en caso contrario.
	 */
	public Message getMessage (String key) {
		if (errors!=null)
			return errors.get(key);
		return null;
	}
	
	/**
	 * Método que recupera la coleccion completa de errores en forma de diccionario. 
	 * @return Colección de pares (clave, Message).
	 */
	public Map<String, Message> getErrors(){
		return errors;
	}
	
	/**
	 * Método que recupera las claves asignadas a los errores existentes. 
	 * @return Colección de claves utilizadas para identificar cada mensaje de error guardado.
	 */
	public Set<String> getKeys() {
		return errors.keySet();
	}
	
	/**
	 * Método que recupera el total de errores existentes.
	 * @return Total de errores existentes.
	 */
	public int size() {
		return errors.size();
	}
	/**
	 * Método que comprueba si la instancia de FormErrors contiene mensajes de error o no.
	 * @return true si hay mensajes definidos, false en cualquier otro caso.
	 */
	public boolean isEmpty() {
		return errors.isEmpty();
	}
}