package edu.uoc.framework.helpers;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * <p>Clase encargada de almacenar todos aquellos valores que sea necesario
 * enviar a la Vista.</p>
 * 
 * <p>Estos valores se identifican mediante una clave a partir de la cual
 * podrán recuperarse posteriormente.</p>
 *  
 * @author Alejandro Marmelo Insua
 */
public class Data 
{
	private Map<String,Object> data;
	
	/**
	 * <p>Constructor que crea una instancia de Data a partir de un 
	 * diccionario existente.</p>
	 * 
	 * @param data Diccionario formado por pares (Clave, Valor) que contiene
	 * los datos que desean enviarse a la Vista.
	 */
	public Data(Map<String, Object> data) {
		this.data = data;
	}
	
	/**
	 * Constructor que crea una instancia de Data partiendo de un
	 * valor y su identificación.
	 * 
	 * @param key Identificador único asociado al valor que quiere almacenarse.
	 * @param value objeto que representa el valor asociado al identificador 'key'.
	 */
	public Data(String key, Object value) 
	{
		this.data = new HashMap<String, Object>();
		put(key, value);
	}
	
	/**
	 * Método que recupera el valor asociado a una clave.
	 * 
	 * @param key Identificador del valor a recuperar.
	 * @return El objeto vinculado con el identificador 'key'.
	 */
	public Object getValue(String key) {
		if(data!=null)
			return data.get(key);
		return null;
	}
	
	/**
	 * Método que recupera el conjunto de claves identificadoras
	 * definidas en el objeto en curso.
	 * 
	 * @return Conjunto de identificadores.
	 */
	public Set<String> keySet() {
		if (data!=null)
			return data.keySet();
		return null;
	}

	/**
	 * Método que añade al diccionario un nuevo un par (clave,valor).
	 * 
	 * @param key Identificador asociado al valor a añadir.
	 * @param value Objeto con tiene el valor vinculado al identificador 'key'.
	 */
	public void put(String key, Object value) {
		if (data!=null)
			data.put(key, value);
	}
	
	/**
	 * Método que recupera el diccionario vinculado al objeto. 
	 * 
	 * @return Diccionario vinculado al objeto.
	 */
	public Map<String,Object> getData() {
		return data;
	}
}