package edu.uoc.framework.interceptor;

import edu.uoc.framework.core.ContextObject;

/**
 * <p>Interfaz que define el contrato que debe seguir toda 
 * clase que quiera utilizarse como interceptor de una
 * acción.</p>
 * <p>Todo interceptor permite llevar a cabo operaciones
 * antes y después de ejecutar la lógica de la acción con la que
 * están vinculados.</p>
 * 
 * <p>Un {@link edu.uoc.framework.action.Action} además de formularios de entrada y recursos a los que
 * redirigir en función de sus resultados, también puede ejecutar cierta lógica antes y después de llevar a cabo una acción.</p>
 * <p>Un interceptor ejecutará su lógica de preProceso antes de que se realice la validación de un {@link edu.uoc.framework.form.Form} 
 * <p>Un interceptor ejecutará su lógica de postProceso después de que el {@link edu.uoc.framework.action.Action} se haya ejecutado.
 * 
 * <p>Ejemplo de definición de interceptores en el fichero de configuración del framework.</p>
 *<pre>
 *{@code
 *<jmvc:interceptors>
 *	<jmvc:interceptor name="interceptor1" class="edu.uoc.interceptors.Interceptor1"/>
 *	<jmvc:interceptor name="interceptor2" class="edu.uoc.interceptors.Interceptor2"/>
 *</jmvc:interceptors>
 *}</pre>
 *
 * <p>Elementos de un Interceptor</p>
 * <ul><li>Name - Atributo obligatorio que indica el nombre o alias del Interceptor. Éste se utilizará al declarar las referencias a interceptores dentro de un Action.</li>
 * <li>class - Atributo obligatorio que indica la clase que implementará el Interceptor.</li></ul>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public interface Interceptor {

	/**
	 * <p>Método que implementará la lógica vinculada con un interceptor antes de que 
	 * la acción vinculada a éste sea ejecutada.</p>
	 * <p>Si una acción tiene una validación de formulario asociada, éste método se ejecutará
	 * antes de llevar a cabo las validaciones asociadas al formulario.</p>
	 * 
	 * @param contextObject Objeto que encapsula los datos de contexto de la petición recibida.
	 * @throws Exception Excepción que indica algún inconveniente durante la ejecución del método.
	 */
	public void doPreprocess(ContextObject contextObject) throws Exception;
	
	/**
	 * <p>Método que implemetará la lógica vinculada con un interceptor después de que
	 * la acción vinculada a éste se haya ejecutado</p>
	 * 
	 * @param contextObject Objeto que encapsula los datos de contexto de la petición recibida.
	 * @throws Exception Excepción que indica algún inconveniente durante la ejecución del método.
	 */
	public void doPostprocess(ContextObject contextObject) throws Exception;
	
}
