package edu.uoc.framework.utilities.I18N;

/**
 * 
 * <p>Clase que abstrae el concepto de mensaje, pudiendo ser o no internacionalizable
 * dependiendo del valor de su propiedad localized.</p>
 * 
 * <p>Si la propiedad localized es true, el campo 'key' representará la clave del mensaje
 * en un paquete de recursos, si por el contrario 'key' es false, este campo contendrá directamente
 * el valor del mensaje.</p>

 * <p>Aquellos mensajes que sean localizados (localized=true) podrán tener un conjunto de valores
 * 'values' asociados, que podrán integrarse al mensaje original si éste contiene marcas del tipo {i},
 *  donde i debe ser un número natural comprendido entre uno y cuatro.</p>
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class Message{

	private String key;
	private Object[] values;
	private boolean localized = true;
	
	/**
	 * Método que construye un Message a partir de un clave y un booleano que indica 
	 * si 'key' es la clave de un mensaje en un paquete de recursos o por el contrario es un literal.
	 * 
	 * @param key Clave del mensaje al que hará referencia la Message.
	 * @param localized booleano que indica si 'key' es la clave de un paquete de recursos o un literal.
	 */
	public Message (String key, boolean localized) {
		this.key = key;
		this.localized = localized;
	}
	
	/**
	 * Método que construye un mensaje localizado a partir de la clave de un mensaje en 
	 * un paquete de recursos.
	 * @param key Clave del mensaje al que hará referencia el Message.
	 */
	public Message (String key) {
		this.key = key;
	}
	
	/**
 	 * <p>Método que construye un mensaje localizado a partir de la clave de un mensaje en 
	 * un paquete de recursos y un valor que reemplazará al token {1} del mensaje recuperado.
	 * 
	 * @param key clave que identifica al mensaje en el paquete de recursos.
	 * @param value1 Valor que reemplazará el token {1} del mensaje recuperado.
	 */
	public Message (String key, Object value1)
	{
		this.key = key;
		this.values = new Object[]{value1};
	}
	
	/**
 	 * <p>Método que construye un mensaje localizado a partir de la clave de un mensaje en 
	 * un paquete de recursos y dos valores
	 * que reemplazarán los tokens {1} y {2} del mensaje recuperado.</p>
	 * 
	 * @param key clave que identifica al mensaje en el paquete de recursos.
	 * @param value1 Valor que reemplazará el token {1} del mensaje recuperado.
	 * @param value2 Valor que reemplazará el token {2} del mensaje recuperado.
	 */
	public Message (String key, Object value1, Object value2)
	{
		this.key = key;
		this.values = new Object[]{value1, value2};
	}
	
	/**
	 * <p>Método que construye un mensaje localizado a partir de la clave de un mensaje en 
	 * un paquete de recursos y tres valores
	 * que reemplazarán los tokens {1}, {2} y {3} del mensaje recuperado.</p>
	 * 
	 * @param key clave que identifica al mensaje en el paquete de recursos.
	 * @param value1 Valor que reemplazará el token {1} del mensaje recuperado.
	 * @param value2 Valor que reemplazará el token {2} del mensaje recuperado.
	 * @param value3 Valor que reemplazará el token {3} del mensaje recuperado.
	 */
	public Message (String key, Object value1, Object value2, Object value3)
	{
		this.key = key;
		this.values = new Object[]{value1, value2, value3};
	}
	
	/**
	 * <p>Método que construye un mensaje localizado a partir de la clave de un mensaje en 
	 * un paquete de recursos y cuatro valores
	 * que reemplazarán los tokens {1}, {2}, {3} y {4} del mensaje recuperado.</p>
	 * 
	 * @param key clave que identifica al mensaje en el paquete de recursos.
	 * @param value1 Valor que reemplazará el token {1} del mensaje recuperado.
	 * @param value2 Valor que reemplazará el token {2} del mensaje recuperado.
	 * @param value3 Valor que reemplazará el token {3} del mensaje recuperado.
	 * @param value4 Valor que reemplazará el token {4} del mensaje recuperado.
	 */
	public Message (String key, Object value1, Object value2, Object value3, Object value4)
	{
		this.key = key;
		this.values = new Object[]{value1, value2, value3, value4};
	}
	
	/**
	 * Método que contruye un mensaje localizado a partir de la clave de un mensaje en
	 * un paquete de recursos y un array de valores.
	 * 
	 * @param key clave que identifica al mensaje en el paquete de recursos.
	 * @param values array de valores a sustituir en el mensaje al que hace referencia 'key'.
	 */
	public Message (String key, Object values[])
	{
		this.key = key;
		this.values = values;
	}
	
	/**
	 * Método que recupera el valor de la propiedad key.
	 * @return valor de la propiedad key.
	 */
	public String getKey() {
		return key;
	}

	/**
	 * Método que establece el valor de la propiedad key.
	 */
	public void setKey(String key) {
		this.key = key;
	}

	/**
	 * Método que recupera el array de valores asociado a la propiedad values. 
	 * @return array de valores.
	 */
	public Object[] getValues() {
		return values;
	}

	/**
	 * Método que permite establecer el valor de la propiedad values.
	 * @param values array de valores.
	 */
	public void setValues(Object[] values) {
		this.values = values;
	}

	/**
	 * Método que recupera el valor de la propiedad localized
	 * @return true si la key del mesanje representa la clave de un fichero de recursos o false si es un String 
	 */
	public boolean isLocalized() {
		return localized;
	}
	
	
}