package edu.uoc.framework.utilities;

import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import edu.uoc.framework.form.Form;

/**
 * Clase que recopila un conjunto de funciones basadas en reflection
 * que son utilizadas por el framework.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ReflectionUtilities 
{
	private static Logger log = Logger.getLogger(ReflectionUtilities.class);
	private static enum METHOD_TYPE {get, set, is};
	
	/**
	 * Método que dado un diccionario de datos implementado mediante una tabla de Hash,
	 * realiza el set de cada clave de la Hash en la propiedad homónima del Form.
	 * 
	 * @param parameters Hash que contiene pares {clave,valor} que representan
	 * los parámetros recogidos tras el envío de un formulario.
	 * 
	 * @param form Cualquier clase que implemente la interfaz Form y sea un Bean que 
	 * conste de atributos de clase homónimos con los nombres que las claves que contiene 'parameters'.
	 */
	public static void mapToForm(Map<String, String[]> parameters, Form form)
	{
		Class<?>[] types = null;
		String[] currentValues = null;
		String attributeName = null;
		Field currentField = null;
		Class<?> aClass = form.getClass();
		Object convertedType = null;
		
		//Se recorren todos los métodos de la clase
		for (Method currentMethod : aClass.getMethods())
		{
			//Si el método es un setter
			if (isSetter(currentMethod))
			{
				types = currentMethod.getParameterTypes();
				attributeName = getAttributeNameFromMethod(currentMethod);
				log.debug("Localizado atributo " + attributeName);
				currentValues = parameters.get(attributeName);
				log.debug("Valores de '" + attributeName + "': " + currentValues);
				if (currentValues!=null)
				{
					try
					{
						currentField = aClass.getDeclaredField(attributeName);
						//El tipo definido para el setter coincide con el tipo del atributo extraído del nombre del setter.
						if (currentField.getType() == types[0])
						{
							//Se convierte del tipo recibido al tipo definido por el usuario según la clase Form
							convertedType = convert(currentValues, currentField.getType());
							if (convertedType!=null)//Se hace el set de la propiedad sólo si hay valor.
								currentMethod.invoke(form,convertedType);
						}
					}
					catch (NoSuchFieldException e) 
					{//Cuando el nombre del campo que se busca no existe
						log.warn(e);
					}
					catch (SecurityException e) 
					{//Cuando se produce una violación de seguridad
						log.warn(e);
					}
					catch (IllegalAccessException e) 
					{//Si el método inaccesible
						log.warn(e);
					}
					catch (IllegalArgumentException e) 
					{//Si la llamada al método se hace con un número incorrecto de parámetros, o una conversion de tipos falla, etc.
						log.warn(e);
					}
					catch (InvocationTargetException e) 
					{//El método envocado ha provocado una excepción.
						log.warn(e);
					}
				}
			}
		}
	}
	
	/**
	 * Método inverso a mapToForm.
	 * 
	 * Dado un objeto de una clase que implemente form y disponga de métodos get
	 * para recuperar sus atributos, genera un Map con cada uno de sus atributos
	 * recuperables mediante un método get.    
	 * 
	 * @param form Clase que implementa form y de la que quieren recuperarse sus atributos. 
	 * @return Map que contiene tantas claves como atributos pudieron recuperarse mediante Getters
	 * de la clase que implementa form.
	 */
	public static Map<String, Object> formToMap (Form form)
	{
		Map<String,Object> data = null;
		Object currentObject = null; 
		String attributeName = null;
		Class<?> aClass = null;
		
		if (form!=null)
		{
			data = new HashMap<String, Object>();
			aClass = form.getClass();
			for (Method currentMethod : aClass.getMethods())
			{
				if (isGetter(currentMethod))
				{
					attributeName = getAttributeNameFromMethod(currentMethod);
					log.debug("Localizado atributo '" + attributeName + "' con método getter asociado");
					try 
					{
						if ((currentObject = currentMethod.invoke(form))!=null)
							data.put(attributeName, currentObject);
					}
					catch (IllegalAccessException e) 
					{//Si el método inaccesible
						log.warn(e);
					}
					catch (IllegalArgumentException e) 
					{//Si la llamada al método se hace con un número incorrecto de parámetros, o una conversion de tipos falla, etc.
						log.warn(e);
					}
					catch (InvocationTargetException e) 
					{//El método envocado ha provocado una excepción.
						log.warn(e);
					}
				}
			}
		}
		return data;
	}
	
	/**
	 * Método que convierte un array de valores de tipo String
	 * en un objeto de la clase especificada por 'target'.
	 *  
	 * @param source Array que contiene los elementos que desean almacenarse en el objeto 'target'.
	 * @param target Objeto destino al que irán a parar los elementos de 'source'.
	 * @return Objeto del tipo indicado por 'target' que contiene los elementos de 'source'.
	 */
	private static Object convert (String[] source, Class<?> target)
	{
		Object object = null;
		
		if (source!=null)
		{
			if (target.isArray())
			{
				int index = 0;
				Class<?> currentType = target.getComponentType();
				object = Array.newInstance(currentType, source.length);
				for (String currentObject : source) 
				{
					Array.set (object,index,convert(currentObject,currentType));
					index++;
				}
			}
			else
			{
				if (!source[0].isEmpty())
					object = convert(source[0], target);
			}
		}
		else
			object = null;
		
		return object;
	}
	
	/**
	 * Método que convierte un String de oriegen en un objeto del tipo definido por target.
	 * 
	 * @param source String que contiene el dato que se desea convertir.
	 * @param target Clase a la que se desea convertir el tipo 'source'.
	 * @return Objeto almacena el contenido 'source' en un objeto que coincide en tipo con 'target'.  
	 */
	private static Object convert(String source, Class<?> target)
	{
		Object object = null;
		
		if (!target.equals(null))
		{
			if (target.isPrimitive())
			{
				if (target.equals(boolean.class))
					object = Boolean.valueOf(source).booleanValue();
				else if (target.equals(byte.class))
					object = Byte.valueOf(source).byteValue();
				else if (target.equals(char.class))
					object = source.charAt(0);
				else if (target.equals(short.class))
					object = Short.valueOf(source).shortValue();
				else if (target.equals(int.class))
					object = Integer.valueOf(source).intValue();
				else if (target.equals(long.class))
					object = Long.valueOf(source).longValue();
				else if (target.equals(float.class))
					object = Float.valueOf(source).floatValue();
				else if (target.equals(double.class))
					object = Double.valueOf(source).doubleValue();
			}
			else
			{
				if (target.equals(Boolean.class))
					object =  Boolean.valueOf(source);
				else if (target.equals(Byte.class))
					object =  Byte.valueOf(source);
				else if (target.equals(Short.class))
					object =  Short.valueOf(source);
				else if (target.equals(Integer.class))
					object =  Integer.valueOf(source);
				else if (target.equals(Long.class))
					object =  Long.valueOf(source);
				else if (target.equals(Float.class))
					object =  Float.valueOf(source);
				else if (target.equals(Double.class))
					object =  Double.valueOf(source);
				else if (target.equals(String.class))
					object = String.valueOf(source);
			}
		}
		return object;
	}
	
	/**
	 * Método que dado un Method recuperado de una clase,
	 * determina si se trata de un método tipo set propio de un bean.
	 * 
	 * @param method Método a analizar.
	 * @return true si method es un setter, false en cualquier otro caso.
	 */
	private static boolean isSetter(Method method)
	{
		if(!method.getName().startsWith("set"))
			return false;
		if(method.getParameterTypes().length != 1) 
			return false;
		return true;
	}
	
	/**
	 * Método que dado un Method de una clase,
	 * determina si se trata de un método tipo get propio de un bean.
	 * 
	 * @param method Método a analizar.
	 * @return true si method es un getter, false en cualquier otro caso.
	 */
	public static boolean isGetter(Method method)
	{
		  if (!method.getName().startsWith("get") && 
		     (!method.getName().startsWith("is") || !method.getReturnType().equals(boolean.class)))
			  return false;
		  if (method.getParameterTypes().length != 0) 
			  return false;  
		  if (void.class.equals(method.getReturnType()))
				  return false;
		  return true;
	}
	
	/**
	 * Método que dado el nombre de un método extrae el nombre
	 * del atributo al que hace referencia si se trata de un 
	 * método de set o get.
	 * 
	 * @param method Método del que quiere extraerse en nombre de la propiedad a la que hace referencia
	 * @return nombre del atributo
	 */
	private static String getAttributeNameFromMethod(Method method)
	{
		String attributeName = null;
		char[] characters = null;
		if (isSetter(method) || isGetter(method))
		{
			if (method.getReturnType().equals(boolean.class))
				attributeName = method.getName().substring(2, method.getName().length());
			else
				attributeName = method.getName().substring(3, method.getName().length());
			
			characters = attributeName.toCharArray();
			characters[0] = (char) Character.toLowerCase(characters[0]);
			attributeName = String.valueOf(characters);
		}
		return attributeName;
	}
	
	/**
	 * Método que construye el nombre que tendría el método get de un bean
	 * a partir del nombre de una propiedad de la clase.
	 * 
	 * @param property Nombre de la propiedad de la que quiere obtener el método get asociado según la convencion Bean.
	 * @return String que representa el nombre del método que en un bean permitiría recuperar el valor de la propiedad 'property'.
	 */
	private static String getGetBeanMethodFromProperty(String property, boolean isBoolean)
	{
		char[] characters = property.toCharArray();
		characters[0] = (char) Character.toUpperCase(characters[0]);
		if (isBoolean)
			return METHOD_TYPE.is.toString() + String.valueOf(characters);
		else
			
			return METHOD_TYPE.get.toString() + String.valueOf(characters);
	}
	
	/**
	 * Método que recibe un object que sigue el estándar bean
	 * junto con una cadena con la propiedad a consultar.
	 * 
	 * Si la propiedad a consultar contiene diversas palabras separadas por puntos,
	 * cada subcadena delimitada entre puntos se considerará como un bean dentro 
	 * del bean anterior. De esta forma se permite recuperar propiedades de objetos
	 * incluidos dentro de otros objetos.
	 * 
	 * @param bean bean base del que quiere recuperarse el valor de una propiedad.
	 * @param property Nombre de la propiedad del bean que quiere recuperarse.
	 * @return valor de la propiedad recuperada del bean.
	 * @throws IllegalAccessException
	 * @throws IllegalArgumentException
	 * @throws InvocationTargetException
	 * @throws NoSuchMethodException
	 * @throws SecurityException
	 * @throws NoSuchFieldException
	 */
	public static Object getValue(Object bean, String property) 
	throws IllegalAccessException, 
		IllegalArgumentException,
		InvocationTargetException,
		NoSuchMethodException, 
		SecurityException,
		NoSuchFieldException
	{
		Method method = null;
		int dotIndex = property.indexOf('.');
		if (dotIndex == -1) //Caso Base
		{
			if ((method = getGetMethod(bean, property))!=null)
				return method.invoke(bean);
			return null;
		}
		else { //Caso recursivo
			method = getGetMethod(bean, property.substring(0,dotIndex));
			return getValue(method.invoke(bean), property.substring(dotIndex+1));
		}
	}

	/**
	 * Método que dado un objeto de una clase y una propiedad de ésta,
	 * recupera el método de la clase que permite recuperar el valor de
	 * dicha propiedad. 
	 * 
	 * @param bean Objeto del que quiere recuperar un Method.
	 * @param property Nombre de la propiedad de la que quiere obtenerse su método get
	 * @return Method que permite recuperar el valor de la propiedad 'property'.
	 * @throws NoSuchMethodException
	 * @throws SecurityException
	 * @throws NoSuchFieldException 
	 */
	private static Method getGetMethod(Object bean, String field) 
	throws NoSuchMethodException,
		SecurityException,
		NoSuchFieldException 
	{
		Field aField = null;
		Class<?> aClass = bean.getClass();
		
		//Se comprueba que el método que se quiere recuperar exista en la clase.
		if ((aField = aClass.getDeclaredField(field))!=null)
		{
			Class<?> type = aField.getType();
			if (type.isPrimitive() && type.equals(boolean.class))
				return aClass.getMethod(getGetBeanMethodFromProperty(field, true));
		}
		return bean.getClass().getMethod(getGetBeanMethodFromProperty(field, false));
	}
	
	/**
	 * Método que comprueba si una Exception es un subtipo de la excepción indicada en 'mappendException'.
	 * 
	 * @param exception Instancia de la excepción que se ha producido. 
	 * @param mappedException package + nombre de una excepción.
	 * @return cierto si exception es una subclase de mappedException, falso en cualquier otro caso.
	 */
	public static boolean hasAncestor (Exception exception, String mappedException)
	{
		boolean hasAncestors = false;
		//Se crea una instancia de la excepción representada por mappedException
		try
		{
			Class<?> mappedType = Class.forName(mappedException);
			exception.getClass().asSubclass(mappedType);
			hasAncestors = true;
		}
		catch (ClassNotFoundException e)
		{
			//La clase a la que hace referencia mappedException no es una clase instanciable
			//por ser desconocida.
			log.warn(e);
		}
		catch (ClassCastException e)
		{
			log.warn(e);
		}
		return hasAncestors;
	}
}