package edu.uoc.framework.view;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;

import org.apache.log4j.Logger;

import edu.uoc.framework.core.ContextObject;
import edu.uoc.framework.core.MessageResourceBundle;
import edu.uoc.framework.exception.ViewException;
import edu.uoc.framework.utilities.Constants;
import edu.uoc.framework.utilities.StringUtils;

/**
 *  Clase que recoge una serie de métodos destinados a 
 *  realizar el envío de datos del contexto actual
 *  al recurso indicado. 
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class ViewUtils 
{
	private static Logger log = Logger.getLogger(ViewUtils.class);
	private static final String FORWARD_TO = "forward.to";
	private static final String FORWARD_TO_EXCEPTION = "forward.to.exception";
	private static final String UNKNOWN_RESOURCE = "unknown.resource";
	
	private static final String REDIRECT_TO = "redirect.to";
	private static final String REDIRECT_TO_EXCEPTION ="redirect.to.exception";

	/**
	 * <p>Método que redirige la respuesta de una petición a un recurso.</p>
	 *
	 * <p>Redirect implica una nueva petición desde el cliente lo que conlleva 
	 * además, la pérdida de los parámetros iniciales de la petición.
	 * Dado que redirect desencadena una nueva petición por parte del cliente
	 * la URL original cambiará.</p>
	 * 
	 * @param contextObject Objeto que encapsula los datos del contexto de la petición.
	 * @param redirectTo nombre del recurso al que debe hacerse un redirect. (Este recurso podrá ser una JSP u otro Action).
	 */
	public static void redirect (ContextObject contextObject, String redirectTo)
	{
		if (!StringUtils.isBlank(redirectTo))
		{
			try
			{
				if (log.isDebugEnabled())
					log.debug(MessageResourceBundle.getMessage(REDIRECT_TO, redirectTo));
				
				contextObject.getResponse().sendRedirect(redirectTo);
			}
			catch (IOException ioe)
			{
				log.error(MessageResourceBundle.getMessage(REDIRECT_TO_EXCEPTION, redirectTo));
				forwardTo(contextObject, ioe);
			}
		}
		else//Si no se ha proporcionado el recurso al que se quiere redirigir, se hace un forward a la página de error.
			forwardTo(contextObject);
	}
	
	/**
	 * <p>Método que ejecuta el forward a un recurso.</p>
	 * <p>La operación de forward mantiene tanto los datos de la request (parámetros y atributos)
	 * como la URL original.</p>
	 * 
	 * @param contextObject Objeto que encapsula los datos del contexto de la petición.
	 * @param resource Recurso al que desea hacer el forward. (Este recurso podrá ser una JSP u otro Action).
	 */
	public static void forwardTo (ContextObject contextObject, String resource)
	{
		if (!StringUtils.isBlank(resource))
		{
			RequestDispatcher requestDispacher = null;
		
			if (log.isDebugEnabled())
				log.debug(MessageResourceBundle.getMessage(FORWARD_TO, resource));
			try
			{
				requestDispacher = contextObject.getRequest().getRequestDispatcher(resource);
				if (requestDispacher!=null)
					requestDispacher.forward(contextObject.getRequest(), contextObject.getResponse());
				else
					forwardTo(contextObject);
			}
			catch (ServletException se)
			{
				log.error(MessageResourceBundle.getMessage(FORWARD_TO_EXCEPTION, resource));
				forwardTo(contextObject, se);
			}
			catch (IOException ioe)
			{
				log.error(MessageResourceBundle.getMessage(FORWARD_TO_EXCEPTION, resource));
				forwardTo(contextObject, ioe);
			}
		}
		else
			forwardTo(contextObject);
	}

	/**
	 * <p>Método que permite hacer un forward a la página de errror del Framework 
	 * cuando, debido a errores en la configuración o a errores en las peticiones
	 * recibidas por el frontController, ha sido imposible determinar su destino.</p> 
	 * 
	 * @param contextObject Objeto que encapsula los datos del contexto de la petición.
	 */
	public static void forwardTo (ContextObject contextObject)	{
		forwardTo(contextObject, new ViewException(MessageResourceBundle.getMessage(UNKNOWN_RESOURCE)));
	}

	/**
	 * <p>Método que realiza un forward, dada una excepción, a la página de error que mostrará los detalles
	 * del error sucedido.</p>
	 * 
	 * @param contextObject Objeto que encapsula los datos del contexto de la petición.
	 * @param exception Excepción sucedida durante el procesamiento de la petición en curso.
	 */
	public static void forwardTo (ContextObject contextObject, Exception exception)
	{
		RequestDispatcher requestDispacher = null;
		try
		{
			contextObject.getRequest().setAttribute(Constants.JavaMVC_Exception, exception);
			requestDispacher = contextObject.getRequest().getRequestDispatcher(Constants.JavaMVC_ExceptionPage);
			requestDispacher.forward(contextObject.getRequest(), contextObject.getResponse());
		}
		catch (ServletException se)
		{
			log.error(MessageResourceBundle.getMessage(FORWARD_TO_EXCEPTION, Constants.JavaMVC_ExceptionPage));
		}
		catch (IOException ioe)
		{
			log.error(MessageResourceBundle.getMessage(FORWARD_TO_EXCEPTION, Constants.JavaMVC_ExceptionPage));
		}
	}
}