package edu.uoc.framework.view.tags;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import edu.uoc.framework.core.MessageResourceBundle;
import edu.uoc.framework.exception.TagException;
import edu.uoc.framework.utilities.ReflectionUtilities;
import edu.uoc.framework.utilities.I18N.I18n.DATE_TIME_FORMAT;
import edu.uoc.framework.utilities.I18N.I18n.I18N_TYPES;

/**
*
* Clase que implementa el tag I18n que podrá utilizarse en una página JSP
* y cuya funcionalidad es la de internacionalizar información en función de su tipo
* pudiendo internacionalizar monedas, fechas, horas, números, etc...
* 
*  Más información en el fichero "JavaMVC.i18n.tld" 
* 
* @author Alejandro Marmelo Insua
*
*/
public class I18nTag  implements Tag 
{
	public static final String TAG_ERROR_I18N_DATA_TYPE_INVALID = "tag.error.i18n.data.type.error";
	
	private PageContext pageContext;
	private Tag parent;
	
	//Nombre del atributo de session que lleva el Locale.
	private String locale;
	
	//Nombre del bean
	private String name;
	
	//Propiedad del bean de la que se extraerá el contenido a internacionalizar.
	private String property;
	
	//Tipo de dato que se va a internacionalizar {currency, date, number, decimal}  
	private String type;
	
	//Formato a utilizar cuando han de internacionalizarse fechas.
	private String dateFormat;
	
	//Formato a utilizar cuando han de internacionalizarse horas.
	private String timeFormat;
	
	//Scope en el que se quiere buscar el bean
	private String scope;
	
	/**
	 * Método que recupera el valor de la propiedad 'pageContext'.
	 * @return {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public PageContext getPageContext() {
		return pageContext;
	}

	/**
	 * Método que establece el valor de la propiedad 'pageContext'.
	 * @param pageContext {@link PageContext} con el valor de la propiedad 'pageContext'.
	 */
	public void setPageContext(PageContext pageContext) {
		this.pageContext = pageContext;
	}

	/**
	 * Método que recupera el valor de la propiedad 'parent'.
	 * @return {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public Tag getParent() {
		return parent;
	}

	/**
	 * Método que establece el valor de la propiedad 'parent'.
	 * @param parent {@link Tag} con el valor de la propiedad 'parent'.
	 */
	public void setParent(Tag parent) {
		this.parent = parent;
	}

	/**
	 * Método que recupera el valor de la propiedad 'locale'.
	 * @return {@link String} con el valor de la propiedad 'locale'.
	 */
	public String getLocale() {
		return locale;
	}
	
	/**
	 * Método que establece el valor de la propiedad 'locale'.
	 * @param locale {@link String} con el valor de la propiedad 'locale'.
	 */
	public void setLocale(String locale) {
		this.locale = locale;
	}

	/**
	 * Método que recupera el valor de la propiedad 'name'.
	 * @return {@link String} con el valor de la propiedad 'name'.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Método que establece el valor de la propiedad 'name'.
	 * @param name {@link String} con el valor de la propiedad 'name'.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Método que recupera el valor de la propiedad 'property'.
	 * @return {@link String} con el valor de la propiedad 'property'.
	 */
	public String getProperty() {
		return property;
	}

	/**
	 * Método que establece el valor de la propiedad 'property'.
	 * @param property {@link String} con el valor de la propiedad 'property'.
	 */
	public void setProperty(String property) {
		this.property = property;
	}

	/**
	 * Método que recupera el valor de la propiedad 'type'.
	 * @return {@link String} con el valor de la propiedad 'type'.
	 */
	public String getType() {
		return type;
	}

	/**
	 * Método que establece el valor de la propiedad 'type'.
	 * @param type {@link String} con el valor de la propiedad 'type'.
	 */
	public void setType(String type) {
		this.type = type;
	}

	/**
	 * Método que recupera el valor de la propiedad 'dateFormat'.
	 * @return {@link String} con el valor de la propiedad 'dateFormat'.
	 */
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	 * Método que establece el valor de la propiedad 'dateFormat'.
	 * @param dateFormat {@link String} con el valor de la propiedad 'dateFormat'.
	 */
	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}

	/**
	 * Método que recupera el valor de la propiedad 'timeFormat'.
	 * @return {@link String} con el valor de la propiedad 'timeFormat'.
	 */
	public String getTimeFormat() {
		return timeFormat;
	}

	/**
	 * Método que establece el valor de la propiedad 'timeFormat'.
	 * @param timeFormat {@link String} con el valor de la propiedad 'timeFormat'.
	 */
	public void setTimeFormat(String timeFormat) {
		this.timeFormat = timeFormat;
	}

	/**
	 * Método que recupera el valor de la propiedad 'scope'.
	 * @return {@link String} con el valor de la propiedad 'scope'.
	 */
	public String getScope() {
		return scope;
	}

	/**
	 * Método que establece el valor de la propiedad 'scope'.
	 * @param scope {@link String} con el valor de la propiedad 'scope'.
	 */
	public void setScope(String scope) {
		this.scope = scope;
	}

	@Override
	/**
	 * Método que procesa el tag de inicio y es invocado por el objeto que implementa la página JSP.
	 */
	public int doStartTag() 
	throws JspException 
	{
		JspWriter out = null;
		Locale locale = null;
		Object bean = null;
		Object value = null;
		String data = null;
		try
		{
			//Se recupera el locale
			locale = TagUtils.recoverLocale(getLocale(), pageContext);
				
			//Se recupera el bean del scope indicado 
			bean = TagUtils.getAttributeFromScope(getName(), getScope(), pageContext);
			
			if (bean!=null)
			{
				//Se recupera el valor que contiene la propiedad del bean.
				value = ReflectionUtilities.getValue (bean, getProperty());
				
				if (value!=null)
				{
					data = TagUtils.internationalizeData(
						value, 
						locale,
						getDataType(getType()),
						getFormat(getDateFormat()),
						getFormat(getDateFormat()));
					
					out = pageContext.getOut();
					out.write(data);
				}
			}
		}
		catch (TagException e)
		{
			throw new JspException(e);
		}
		catch (IllegalAccessException e) 
		{
			throw new JspException(e);
		}
		catch (IllegalArgumentException e) 
		{
			throw new JspException(e);
		}
		catch (InvocationTargetException e) 
		{
			throw new JspException(e);
		}
		catch (NoSuchMethodException e) 
		{
			throw new JspException(e);
		}
		catch (SecurityException e) 
		{
			throw new JspException(e);
		}
		catch (IOException e) 
		{
			throw new JspException(e);
		}
		catch (NoSuchFieldException e) 
		{
			throw new JspException(e);
		}
		//Si el tag contiene contenido éste se ignora.
		return Tag.SKIP_BODY;
	}
	
	private DATE_TIME_FORMAT getFormat(String format)
	{
		if ("Default".equalsIgnoreCase(format))
			return DATE_TIME_FORMAT.Default;
		else if ("Medium".equalsIgnoreCase(format))
			return DATE_TIME_FORMAT.Medium;
		else if ("Long".equalsIgnoreCase(format))
			return DATE_TIME_FORMAT.Long;
		else if ("Full".equalsIgnoreCase(format))
			return DATE_TIME_FORMAT.Full;
		else 
			return DATE_TIME_FORMAT.Short;
	}
	
	private I18N_TYPES getDataType (String type)
	throws TagException
	{
		if ("Currency".equalsIgnoreCase(type))
			return I18N_TYPES.Currency;
		else if ("DateTime".equalsIgnoreCase(type))
			return I18N_TYPES.Date_Time;
		else if ("Date".equalsIgnoreCase(type))
			return I18N_TYPES.Date;
		else if ("Time".equalsIgnoreCase(type))
			return I18N_TYPES.Time;
		else if ("Decimal".equalsIgnoreCase(type))
			return I18N_TYPES.Decimal;
		else if ("Number".equalsIgnoreCase(type))
			return I18N_TYPES.Number;
		else
			throw new TagException(MessageResourceBundle.getMessage(TAG_ERROR_I18N_DATA_TYPE_INVALID, type));
	}
	
	@Override
	/**
	 * Método que procesa el tag de fin y es invocado por el objeto que implementa la página JSP en todos los handlers de etiquetas.
	 */
	public int doEndTag()
	throws JspException {
		return Tag.EVAL_PAGE;
	}
	
	/**
	 * Método que libera los recursos utilizados por el Tag.
	 */
	public void release() 
	{
		locale = null;
		name = null;
		property = null;
		type = null;
		dateFormat = null;
		timeFormat = null;
		scope = null;
	}
}