package edu.uoc.phonebook.business;

import java.util.Date;
import java.util.List;

import edu.uoc.phonebook.business.dataobject.Contact;

public interface IIntegrationContactController {

	/**
	 * Método que crea un contacto
	 * 
	 * @param contact {@link Contact}
	 * @return true si el contacto se ha creado correctamente, false en caso contrario.
	 */
	public boolean create (Contact contact);
	
	/**
	 * Método que actualiza un contacto
	 * 
	 * @param contact {@link Contact}
	 * @return true si el contacto se ha actualizado correctamente, false en caso contrario.
	 */
	public boolean update (Contact contact);
	
	/**
	 * Método que elimina un contacto de un usuario.
	 * 
	 * @param contactId identificador del contacto.
	 * @param userId identificador del usuario.
	 * @return true si el contacto se ha eliminado, false en caso contrario.
	 */
	public boolean delete (Integer contactId, Integer userId);
	
	/**
	 * Método que carga el contacto con id 'contactId' del usuario con id 'userId'. 
	 * 
	 * @param contactId Identificador del contacto.
	 * @param userId Identificador del usuario.
	 * @return {@link Contact}
	 */
	public Contact loadById (Integer contactId, Integer userId);

	/**
	 * Método que recupera una colección de contactos a partir de unos criterios de búsqueda. 
	 * 
	 * @param name nombre por el que deben empezar los contactos a buscar.
	 * @param phone teléfono del contacto a buscar
	 * @param town ciudad del contacto a buscar
	 * @param zipCode código postal del contacto a buscar
	 * @param startDate Fecha a partir de la cual se buscarán contactos cuya fecha de nacimiento sea mayor o igual a ella.
	 * @param endDate Fecha hasta la que se buscarán contactos cuya fecha de nacimiento sea menor o igual a ella.
	 * @param userId Identificador del usuario de la aplicación que quiere buscar los contactos. 
	 * @return colección de contactos que cumplen con los criterios de búsqueda.
	 */
	public List<Contact> find (String name, String phone, String town, String zipCode, Date startDate, Date endDate, Integer userId);
	
	/**
	 * Método que recupera los datos de un contacto a partir de su email y del usuario que lo busca.
	 * 
	 * @param email dirección de correo electrónio
	 * @param userId Identificador del usuario de la aplicación que quiere busca el contacto. 
	 * @return {@link Contact}
	 */
	public Contact getContactByEmail(String email, Integer userId);
	
	/**
	 * Método que recupera un contacto a partir del teléfono y del id del usuario que lo busca.  
	 * 
	 * @param phone teléfono 
	 * @param userId Identificador del usuario de la aplicación que quiere buscar los contactos. 
	 * @return {@link Contact}
	 */
	public Contact getContactByPhone(String phone, Integer userId);
}