package edu.uoc.phonebook.business.controllers;

import java.text.ParseException;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import edu.uoc.phonebook.business.dataobject.Contact;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidDateException;
import edu.uoc.phonebook.exceptions.PhoneAlreadyExistsException;
import edu.uoc.phonebook.integration.controllers.IntegrationContactController;
import edu.uoc.phonebook.presentation.IBusinessContactController;
import edu.uoc.phonebook.utils.Utils;
import edu.uoc.phonebook.utils.Utils.FormatosFecha;

public class BusinessContactController implements IBusinessContactController {

	private IntegrationContactController integrationContactController = null;
	
	public BusinessContactController() {
		integrationContactController = new IntegrationContactController();
	}
	
	public boolean createContact(
			String name,
			String surname,
			String address,
			String phone,
			String zipCode,
			String town,
			String email,
			String strBirthDate,
			Integer userId)
	throws EmailAlreadyExistsException,
	PhoneAlreadyExistsException,
	InvalidDateException
	{
		Date birthDate = null;
		Contact contact = null;
		boolean contactCreated = false;
		
		//Comprobar que no exista otro usuario con el mismo teléfono.
		if (integrationContactController.getContactByEmail(email, userId)!=null)
			//Error ya existe un usuario con ese email.
			throw new EmailAlreadyExistsException();
		else if (integrationContactController.getContactByPhone(phone, userId)!=null)
			//Error ya existe un usuario con ese teléfono.
			throw new PhoneAlreadyExistsException();
		else
		{
			try 
			{
				//Se comprueba si la fecha de nacimiento se ha informado para transformarla en un tipo Date.
				if (!StringUtils.isBlank(strBirthDate))
					birthDate = Utils.parseDate(strBirthDate, Utils.FormatosFecha.DMY);
				
				contact = new Contact();
				contact.setName(name);
				contact.setSurname(surname);
				contact.setAddress(address);
				contact.setPhone(phone);
				contact.setZipCode(zipCode);
				contact.setTown(town);
				contact.setEmail(email);
				contact.setBirthdate(birthDate);
				contact.setUserId(userId);
				
				//Se le pide al controlador de la capa de integración que de de alta un nuevo contacto.
				contactCreated = integrationContactController.create(contact);
			}
			catch (ParseException pe)
			{
				throw new InvalidDateException();
			}
		}
		return contactCreated;	
	}
	
	public boolean updateContact(
			String name,
			String surname,
			String address,
			String phone,
			String zipCode,
			String town,
			String email,
			String strBirthDate,
			Integer contactId,
			Integer userId)
	throws EmailAlreadyExistsException,
	PhoneAlreadyExistsException,
	InvalidDateException
	{
		Contact contact = null;
		boolean contactUpdated = false;
		//Se le pide al controlador de la capa de integración que recupere los datos de un contacto
		//a partir de su contactId y su userId.
		if ((contact = integrationContactController.loadById(contactId, userId))!=null)
		{
			//Se comprueba si el email se ha cambiado con respecto al original del contacto
			if (!StringUtils.equals(email, contact.getEmail()))
			{
				//Se le pide al controlador de la capa de integración que recupere el contacto
				//a partir de su email y el userId.
				if (integrationContactController.getContactByEmail(email, userId)!=null)
					throw new EmailAlreadyExistsException();
			}
			//Se comprueba si el teléfono se ha cambiado con respecto al original del contacto
			if (!StringUtils.equals(phone, contact.getPhone()))
			{
				//Se le pide al controlador de la capa de integración que recupere el contacto
				//a partir de su teléfono y el userId.
				if (integrationContactController.getContactByPhone(phone, userId)!=null)
					throw new PhoneAlreadyExistsException();
			}
			try
			{
				contact.setName(name);
				contact.setSurname(surname);
				contact.setAddress(address);
				contact.setPhone(phone);
				contact.setEmail(email);
				contact.setTown(town);
				contact.setZipCode(zipCode);
				contact.setBirthdate(Utils.parseDate(strBirthDate, FormatosFecha.DMY));
				contact.setContactId(contactId);
				//Se le pide al controlador de la capa de integración que actualice los datos del contacto.
				contactUpdated = integrationContactController.update(contact);
			}
			catch (ParseException pe)
			{
				throw new InvalidDateException();
			}
		}
		return contactUpdated;
	}
	
	public List<Contact> findContacts(String name, String phone, String zipCode, String town, String startDate, String endDate, Integer userId)
	throws InvalidDateException
	{
		Date sDate = null;
		Date eDate = null;
		
		//Se comprueba si la fecha de inicio se ha informado
		if (!StringUtils.isBlank(startDate))
		{
			try 
			{
				//Se convierte el string que representa la fecha de inicio a un objeto Date que representa esa fecha.
				sDate = Utils.parseDate(startDate, FormatosFecha.DMY);
			}
			catch (ParseException e)
			{
				throw new InvalidDateException();
			}
		}
		
		//Se comprueba si la fecha de fin se ha informado
		if (!StringUtils.isBlank(endDate))
		{
			try 
			{
				//Se convierte el string que representa la fecha de fin a un objeto Date que representa esa fecha.
				eDate = Utils.parseDate(endDate, FormatosFecha.DMY);
			}
			catch (ParseException e)
			{
				throw new InvalidDateException();
			}
		}
		
		//Se le pide al controlador de la capa de integración que recupere la colección de contactos que cumplan con los 
		//criterios de búsqueda.
		return (List<Contact>) integrationContactController.find(name, phone, town, zipCode, sDate, eDate, userId);
	}
	
	public Contact getContact(Integer contactId, Integer userId) 
	{
		//Se le pide al controlador de la capa de integración que recupere un contacto a partir del 
		//contactId y el userId.
		return integrationContactController.loadById(contactId, userId);
	}
	
	public boolean deleteContact (Integer contactId, Integer userId) {
		return integrationContactController.delete(contactId, userId);
	}
}