package edu.uoc.phonebook.presentation;

import java.util.List;

import edu.uoc.phonebook.business.dataobject.Contact;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidDateException;
import edu.uoc.phonebook.exceptions.PhoneAlreadyExistsException;

/**
 * Interfaz que define los métodos de negocio necesarios para llevar a cabo las operaciones de la capa de presentación referentes a los contactos.
 * 
 * @author Alejandro Marmelo Insua.
 *
 */
public interface IBusinessContactController {
	
	/**
	 * Método que lleva a cabo la creación de un contacto a partir de los datos recibidos.  
	 * 
	 * @param name Nombre del contacto.
	 * @param surname Apellidos del contacto.
	 * @param address Dirección del contacto.
	 * @param phone Teléfono del contacto.
	 * @param zipCode Código postal del contacto.
	 * @param town Ciudad del contacto.
	 * @param email Correo electrónico del contacto.
	 * @param strBirthDate Fecha de nacimiento del contacto.
	 * @param userId Identificador del usuario de la aplicación que quiere crear un nuevo contacto. 
	 * @return true si el contacto se ha creado correctamente, false en cualquier otro caso.
	 * @throws EmailAlreadyExistsException Sucede cuando el correo electrónico informado corresponde a otro contacto.
	 * @throws PhoneAlreadyExistsException Sucede cuando el teléfono informado corresponde a otro contacto.
	 * @throws InvalidDateException Sucede cuando la fecha de nacimiento informada no es válida.
	 */
	public boolean createContact(
			String name,
			String surname,
			String address,
			String phone,
			String zipCode,
			String town,
			String email,
			String strBirthDate,
			Integer userId)
	throws EmailAlreadyExistsException,
	PhoneAlreadyExistsException,
	InvalidDateException;
	
	/**
	 * Método que lleva a cabo la actualización de un contacto.  
	 * 
	 * @param name Nombre del contacto.
	 * @param surname Apellidos del contacto.
	 * @param address Dirección del contacto.
	 * @param phone Teléfono del contacto.
	 * @param zipCode Código postal del contacto.
	 * @param town Ciudad del contacto.
	 * @param email Correo electrónico del contacto.
	 * @param strBirthDate Fecha de nacimiento del contacto.
	 * @param userId Identificador del usuario de la aplicación que quiere actualizar el contacto. 
	 * @return true si la actualización se ha llevado a cabo correctamente, false en caso contrario.
	 * @throws EmailAlreadyExistsException Sucede cuando el correo electrónico informado corresponde a otro contacto.
	 * @throws PhoneAlreadyExistsException Sucede cuando el teléfono informado corresponde a otro contacto.
	 * @throws InvalidDateException Sucede cuando la fecha de nacimiento informada no es válida.
	 */
	public boolean updateContact(
			String name,
			String surname,
			String address,
			String phone,
			String zipCode,
			String town,
			String email,
			String strBirthDate,
			Integer contactId,
			Integer userId)
	throws EmailAlreadyExistsException,
	PhoneAlreadyExistsException,
	InvalidDateException;
	
	/**
	 * Método que busca aquellos contactos del usuario que cumplan con los criterios de búsqueda informados.
	 * 
	 * @param name cadena que representa el nombre o parte de éste a partir del cual deben empezar los nombre de los contactos. 
	 * @param phone Número de teléfono del contacto.
	 * @param zipCode Código postal del contacto.
	 * @param town Ciudad del contacto.
	 * @param startDate Fecha a partir de la cual se buscarán contactos cuya fecha de nacimiento sea mayor o igual a ella.
	 * @param endDate Fecha hasta la que se buscarán contactos cuya fecha de nacimiento sea menor o igual a ella.
	 * @param userId Identificador del usuario de la aplicación que quiere realizar la búsqueda. 
	 * @return Colección de contactos que cumplen con los criterios de búsqueda indicados.
	 * @throws InvalidDateException Sucede cuando alguno de los intervalos de fechas indicados no son válidos.
	 */
	public List<Contact> findContacts(
			String name, 
			String phone,
			String zipCode,
			String town,
			String startDate,
			String endDate, 
			Integer userId)
	throws InvalidDateException;
	
	/**
	 * Método que permite recuperar los datos de un contacto.
	 * 
	 * @param contactId Identificador del contacto.
	 * @param userId Identificador del usuario de la aplicación que quiere recuperar los datos del contacto.
	 * @return {@link Contact} con los datos del contacto, null en caso de que no exista.
	 */
	public Contact getContact(Integer contactId, Integer userId);
	
	/**
	 * Método que permite a un usuario eliminar un contacto de su agenda.
	 * 
	 * @param contactId Identificador del contacto a eliminar.
	 * @param userId Identificador del usuario de la aplicación que quiere recuperar los datos del contacto.
	 * @return true si el contacto se ha eliminado correctamente, false en caso contrario.
	 */
	public boolean deleteContact (Integer contactId, Integer userId);
}
