package edu.uoc.phonebook.presentation;

import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidCredentialException;
import edu.uoc.phonebook.exceptions.InvalidPasswordException;
import edu.uoc.phonebook.exceptions.UserAlreadyExistsException;

/**
 * Interfaz que define los métodos de negocio necesarios para llevar a cabo las operaciones de la capa de presentación referentes a los usuarios.
 * 
 * @author Alejandro Marmelo Insua.
 *
 */
public interface IBusinessUserController 
{
	/**
	 * Método que lleva a cabo la recuperación de los datos de un usuario del sistema.  
	 * 
	 * @param username nombre de usuario
	 * @param password clave de acceso al sistema
	 * @return {@link User} con los datos del usuario que coinciden con las credenciales proporcionadas.
	 */
	public User getUser(String username, String password);
	
	/**
	 * Método que permite la creación de un usuario en el sistema.
	 * 
	 * @param username nombre de usuario
	 * @param password clave de acceso
	 * @param email correo electrónico
	 * @return true si el alta se ha llevado a cabo correctamente, false en caso contrario.
	 * @throws InvalidPasswordException Sucede cuando la clave de acceso proporcionada no cumple los mínimos de seguridad.
	 * @throws UserAlreadyExistsException Sucede cuando el username proporcionado ya corresponde a otro usuario del sistema.
	 * @throws EmailAlreadyExistsException Sucede cuando el email proporcionado ya pertenece a otro usuario del sistema.
	 */
	public boolean createUser (
			String username,
			String password,
			String email) 
	throws InvalidPasswordException,
	UserAlreadyExistsException,
	EmailAlreadyExistsException;
	
	/**
	 * Método que permite comprobar si un password es válido.
	 * 
	 * @param password clave de acceso a validar.
	 * @return true si la clave de acceso es válida, false en caso contrario.
	 */
	public boolean checkValidPassword (String password);
	
	/**
	 * Método que permite actualizar la clave de acceso de un usuario del sistema.
	 * 
	 * @param username Nombre de usuario
	 * @param oldPassword Clave de acceso actual
	 * @param newPassword Nueva clave de acceso
	 * @return true si la actualización se ha llevado a cabo correctamente, false en caso contrario.
	 * @throws InvalidCredentialException Sucede cuando las credenciales de acceso no corresponden con ningun usuario.
	 * @throws InvalidPasswordException Sucede cuando la clave de acceso proporcionada no cumple los mínimos de seguridad.
	 */
	public boolean updatePassword (
			String username,
			String oldPassword,
			String newPassword)
	throws InvalidCredentialException,
	InvalidPasswordException;
	
	/**
	 * Método que permite recuperar los datos de un usuario a partir de su email.
	 * 
	 * @param email correo electronico  
	 * @return {@link User} con los datos del usuario recuperado, null en caso de que no exista.
	 */
	public User getUserByEmail (String email);
}