package edu.uoc.phonebook.presentation.actions;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessContactController;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidDateException;
import edu.uoc.phonebook.exceptions.PhoneAlreadyExistsException;
import edu.uoc.phonebook.presentation.IBusinessContactController;
import edu.uoc.phonebook.presentation.forms.ContactForm;

/**
 * Acción que lleva a cabo la creación de un contacto en la agenda de un usuario.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class CreateContact extends ActionBase 
{
	private static Logger log = Logger.getLogger(CreateContact.class);
	
	@Override
	public ActionResponse doExecute(User user)
	throws Exception
	{
		ActionResponse actionResponse = null;
		ContactForm contactForm = (ContactForm)getForm();
		IBusinessContactController businessContactController = new BusinessContactController();
		
		try
		{
			//Se solicita al controlador de la capa de negocio que lleve a cabo la creación del contacto
			//a partir de los datos recuperados del formulario recibido.
			if (businessContactController.createContact(
					contactForm.getName(), 
					contactForm.getSurname(),
					contactForm.getAddress(),
					contactForm.getPhone(),
					contactForm.getZipCode(),
					contactForm.getTown(),
					contactForm.getEmail(),
					contactForm.getBirthDate(),
					user.getUserId()))
			{
				//Se crea una respuesta que se mostrará en la vista con nombre 'ok' a la que se le envía un mensaje
				//localizado que indica que la creación del contacto ha sido satisfactoria.
				actionResponse = new ActionResponse("success", new Message("contact.creation.successful"), OK);
				log.info("El contacto se ha creado correctamente");
			}
			else
			{
				//Si la creación del contacto no se ha podido llevar a cabo.
				log.error("Imposible realizar el alta del contacto en base de datos");
				//Se añade un mensaje de error informado del problema
				addErrorMessage("errors",new Message("contact.creation.error"));
				//Se crea una respuesta que apuntará a la vista con nombre 'error'.
				actionResponse = new ActionResponse(ERROR);
			}
		}
		catch (EmailAlreadyExistsException e)
		{
			/* Se ha producido un error que indica que ya existe otro contacto en el sistema con el mismo email
			 * Se añade un mensaje a la respuesta y se redirecciona a la vista de error.*/
			log.error("Ya existe un contacto con el email " + contactForm.getEmail());
			addErrorMessage("errors",new Message("contact.email.exists",contactForm.getEmail()));
			actionResponse = new ActionResponse("contact", contactForm, ERROR);
		}
		catch (PhoneAlreadyExistsException e)
		{
			/* Se ha producido un error que indica que ya existe otro contacto en el sistema con el mismo teléfono
			 * Se añade un mensaje a la respuesta y se redirecciona a la vista de error.*/
			log.error("Ya existe un contacto con el teléfono " + contactForm.getPhone());
			addErrorMessage("errors",new Message("contact.phone.exists", contactForm.getPhone()));
			actionResponse = new ActionResponse("contact", contactForm, ERROR);
		}
		catch (InvalidDateException e)
		{
			/* Se ha producido un error que indica que la fecha de nacimiento del contacto no es válida
			 * Se añade un mensaje a la respuesta y se redirecciona a la vista de error.*/
			log.error("La fecha de nacimiento del contacto " + contactForm.getBirthDate() + " no sigue el patrón [dd-mm-yyyy]");
			addErrorMessage("errors",new Message("contact.birthDate.error"));
			actionResponse = new ActionResponse("contact", contactForm, ERROR);
		}
		return actionResponse;
	}
}