package edu.uoc.phonebook.presentation.actions;

import org.apache.log4j.Logger;

import edu.uoc.framework.action.ActionResponse;
import edu.uoc.framework.utilities.I18N.Message;
import edu.uoc.phonebook.business.controllers.BusinessContactController;
import edu.uoc.phonebook.business.dataobject.Contact;
import edu.uoc.phonebook.business.dataobject.User;
import edu.uoc.phonebook.exceptions.EmailAlreadyExistsException;
import edu.uoc.phonebook.exceptions.InvalidDateException;
import edu.uoc.phonebook.exceptions.PhoneAlreadyExistsException;
import edu.uoc.phonebook.presentation.IBusinessContactController;
import edu.uoc.phonebook.presentation.forms.ContactForm;

/**
 * Acción encargada de llevar a cabo la actualización de los datos de un contacto.
 * 
 * @author Alejandro Marmelo Insua
 *
 */
public class UpdateContact extends ActionBase 
{
	private static Logger log = Logger.getLogger(UpdateContact.class);
	
	@Override
	public ActionResponse doExecute(User user)
	throws Exception
	{
		ActionResponse actionResponse = null;
		Contact contact = null;
		IBusinessContactController businessContactController = new BusinessContactController();
		ContactForm contactForm = (ContactForm) getForm();
		
		try
		{
			//Se le solicita al controlador de la capa de negocio que lleve a cabo la actualización 
			//de los datos de un contacto a partir de los datos recuperados del formulario enviado.
			if (businessContactController.updateContact(
				contactForm.getName(), 
				contactForm.getSurname(),
				contactForm.getAddress(),
				contactForm.getPhone(),
				contactForm.getZipCode(),
				contactForm.getTown(),
				contactForm.getEmail(),
				contactForm.getBirthDate(),
				contactForm.getContactId(),
				user.getUserId()))
			{
				//Si los datos del contacto se han actualizado correctamente, se piden los datos del contacto
				//para enviarlos a la vista.
				contact = businessContactController.getContact(contactForm.getContactId(), user.getUserId());
				setRequestAttribute("success", new Message("contact.update.successful"));
				actionResponse = new ActionResponse("contact", contact, OK);
				log.info("Se han actualizado los datos del contacto " + contactForm.getContactId());
			}
			else
			{
				/* Los datos del contacto no han podido actualizarse.
				 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
				log.error("Ha sido imposible realizar la actualización de la información del contacto " + contactForm.getContactId());
				addErrorMessage("errors",new Message("contact.update.error"));
				actionResponse = new ActionResponse("contact", contactForm, ERROR);
			}
		}
		catch (EmailAlreadyExistsException eae)
		{
			/* La agenda del usuario ya dispone de un contacto con el mismo email.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("Ya existe un contacto en la agenda con el email " + contactForm.getEmail());
			addErrorMessage("errors",new Message("contact.email.exists", contactForm.getEmail()));
			actionResponse = new ActionResponse("contact", contactForm, ERROR);
		}
		catch (PhoneAlreadyExistsException pae)
		{
			/* La agenda del usuario ya dispone de un contacto con el mismo teléfono.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("Ya existe un contacto en la agenda con el teléfono " + contactForm.getPhone());
			addErrorMessage("errors",new Message("contact.phone.exists", contactForm.getPhone()));
			actionResponse = new ActionResponse("contact", contactForm, ERROR);
		}
		catch (InvalidDateException ide)
		{
			/* La fecha de nacimiento proporcionada no es válida.
			 * Se añade un mensaje de error a la respuesta y se redirecciona a la vista de error.*/
			log.error("La fecha de nacimiento '" + contactForm.getBirthDate() + "' no sigue el patrón dd-mm-yyyy");
			addErrorMessage("errors",new Message("contact.birthDate.error"));
			actionResponse = new ActionResponse("contact", contactForm, ERROR);
		}
		return actionResponse;
	}
}